/** (c) Brendan Macmillan 2001, licensed under GNU's GPL 2 */
/** (Also available for non-GPL, commercial use, for fee) */
/** Website: http://www.csse.monash.edu.au/~bren/JSX
/** Website: http://freshmeat.net/projects/jsx


/** ===========================================================================
	* ObjIn - (could be called XMLInputOutputStream)
	*	=====
	* Input/deserialization.
	* Class-specific readObject() and writeObject() serialization.
	* USAGE:
	* 	java JSX.ObjIn < test.xml
	*
	* There are many possible calls to the Serialization API; but they are
	* all callbacks, from within readObject(), and so in a sense are invisible.
	**/

//package ObjOutDev;
package JSX;
//import JSX.*;	//use this, not "package", if truly a "wrapper"
import java.io.*;	//Stream

public class ObjIn extends ObjectInputStream {
	static final boolean DEBUG = false;
	//static final boolean DEBUG = true;
	static final boolean TRACE = false;
	//static final boolean TRACE = true;
	static final boolean NONSTREAM = false;
	//static final boolean NONSTREAM = true;


	/**	==========================================================================
		* CONSTRUCTORS
		* ============
		**/
	/** -------------------------------------------------------------------------
		* ObjIn(in) - core constructor (others call it).
		* ------
		**/
	private XMLDeserialize d;
	public ObjIn(Reader in) throws IOException {
		super();	//sets flag, so readObject() defers to overrideReadObject()
		d = new XMLDeserialize(in);
		d.setArg(new Object[] {this});
			//d.readObjectArglist = new Object[] {this};
		if (TRACE) System.err.println("ObjIn(inReader)");
	}

	/** -------------------------------------------------------------------------
		* ObjIn(in)
		* ------
		* Convenience: wraps as a Reader.
		**/
	public ObjIn(InputStream in) throws IOException {	//concession to convenience
		this(new InputStreamReader(in));
		if (TRACE) System.err.println("ObjIn(in)");
	}

	/** -------------------------------------------------------------------------
		* ObjIn()
		* ------
		* Default to System.in (so can redirect files in etc).
		*	Incidentally: SecurityException is RunTime, so needn't explicitly throw.
		**/
	public ObjIn() throws IOException, SecurityException {
		this(System.in);		//default
		if (TRACE) System.err.println("ObjIn()");
	}



	/**	==========================================================================
		* METHODS
		*	=======
		**/
	/** -------------------------------------------------------------------------
		* reset()
		* -------
		*	To reset alias table, mainly.
		* Not needed: resets automatically upon each new invocation.
		* Consider: option to change this when common objects are repeatedly
		* 	messaged.
		* Question: is this in OIS API?
		**/
	public void reset() {	d.reset(); }	//for aliases
	public void close() {	d.close(); }

	/** -------------------------------------------------------------------------
		* readObjectOverride() -- different arg/return form output version
		* --------------------
		*	The heart.  ObjectInputStream.readObject() defers to this, if flag set.
		* Consider: only IOException should be thrown.
		**/
//maybe should make a general loader for this stuff?  Eg MagicClass routines?
	static java.lang.reflect.Method doValidationsMethod;
	static { //class initializer
		try {
			doValidationsMethod = java.io.ObjectInputStream.class.getDeclaredMethod("doValidations", new Class[]{});
			doValidationsMethod.setAccessible(true);
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
	}
	protected Object readObjectOverride() throws IOException {
		if (d.invocationCount>0)	//ie if already inside
			defaultReadObject();	//false = don't read in fields
		if (TRACE) System.err.println("readObjectOverride()");
		Object o = null;
		try {
			o = d.deserialize();
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (TRACE) System.err.println("Object returned:" + o);
		try {
			doValidationsMethod.invoke(this, new Object[]{});	//and that's it!
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (java.lang.reflect.InvocationTargetException e) {
			e.printStackTrace();
		}
		return o;
	}

	/** -------------------------------------------------------------------------
		* defaultReadObject()
		* -----------------
		* Not implemented: presently, default stuff is written anyway.
		**/
	public void defaultReadObject() throws IOException {
		if (TRACE) System.err.println("defaultReadObject()");
		//This does nothing!!
		//What if a class *depends* on these values *not* being read in?
		//Can we simulate the effect by not writing if defaultWriteObject() not
		//called?  That would help!!  Classes then *shouldn't* create this problem!

		d.defaultReadObject();	//forward it.
	}
	public void readFully(byte[] dst) throws IOException {
			//the "fully" is to do with blocking, not the array being full
		byte[] src = (byte[]) readObjectOverride();	//so needn't catch exception
		System.arraycopy(src, 0, dst, 0, src.length);	//err if dst too short
		//this is the odd thing you have to do in Java, to return by reference.
		//it would be nice if we could point the dst at the src instead...
		//probably *can* do this with the private native trick... ;-)
	}
	public int readInt() throws IOException {
		defaultReadObject();	//false = don't read in fields
		return Integer.parseInt(d.getPrimString());
	}
	public float readFloat() throws IOException {
		defaultReadObject();	//false = don't read in fields
		//return Float.parseFloat(d.getPrimString());
		return ParseUtilities.parseFloat(d.getPrimString());
	}
		//we are stricter than Java's own parse of Boolean...
		//(Suggest: This could go into ParseUtilities)
	public boolean readBoolean() throws IOException {
		defaultReadObject();	//false = don't read in fields
		return ParseUtilities.parseBoolean(d.getPrimString());
	}
	public byte readByte() throws IOException {
		defaultReadObject();	//false = don't read in fields
		return Byte.parseByte(d.getPrimString());
	}
	public char readChar() throws IOException {
		defaultReadObject();	//false = don't read in fields
		//XML...
		return d.getPrimString().charAt(0);	//Question: is there a nicer parse?
	}
	public double readDouble() throws IOException {
		defaultReadObject();	//false = don't read in fields
		//return Double.parseDouble(d.getPrimString());
		return ParseUtilities.parseDouble(d.getPrimString());
	}
	//Float - see above
	//Int - see above
	public short readShort() throws IOException {
		defaultReadObject();	//false = don't read in fields
		return Short.parseShort(d.getPrimString());
	}
		//Also thanks to Andy Thomas - odd that both were left out!
	public long readLong() throws IOException {
		defaultReadObject();	//false = don't read in fields
		return Long.parseLong(d.getPrimString());
	}

/* deprecated in java.io.ObjectInputStream
	public String readLine() throws IOException {
		System.err.println("readLine() not implemented");
		return null;
	}
*/
	public String readUTF() throws IOException {
		defaultReadObject();	//false = don't read in fields
		System.err.println("getPrimString() is correct, not readObject()");
		return d.getPrimString();
/*
		try {
			return (String) readObject();  //if it's not String, then an error!
		} catch (ClassNotFoundException e) {
			e.printStackTrace();	//shouldn't be needed: it should be a String.
		}
		return null;
*/
	}

	/**	--------------------------------------------------------------------------
		* Non-stream API
		* --------------
		* This is for stuff that is not actually reading (or writing, for ObjOut).
		* Eg: validation
		**/

/** registerValidation: we simply let this fall through to the superclass
	* implementation.  All we need do is call "doValidation()" when the entire
	* object graph is done.
	* (1). Where do we put it, so we know when it is done?
	* (2). How do we call it with reflection?
	*/
	static java.lang.reflect.Field recursionDepthField;
	static { //class initializer
		try {
			recursionDepthField = java.io.ObjectInputStream.class.getDeclaredField("recursionDepth");
			recursionDepthField.setAccessible(true);
		} catch (NoSuchFieldException e) {
			e.printStackTrace();
		}
	}
	{ //instance initializer - need an instance of field, to set it
		try {
			recursionDepthField.setInt(this, 1);	//just a hack, to use superclass
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
	}
	public void registerValidation(ObjectInputValidation obj, int prio) throws InvalidObjectException, NotActiveException {
		//throw new RuntimeException("registerValidation() invoked - not implemented");
		if (d.invocationCount==0)	//ie not active
			throw new NotActiveException("JSX.ObjIn: readObject not Active");
		else
			super.registerValidation(obj, prio);	//forward it
	}

	protected  Class resolveClass(ObjectStreamClass v) {
		throw new RuntimeException("resolveClass() invoked - not implemented");
	//	return null;
	}
	protected  Object resolveObject(Object obj) {
		throw new RuntimeException("resolveObject() invoked - not implemented");
	//	return null;
	}
	public int skipBytes(int len) {
		throw new RuntimeException("skipBytes() invoked - not implemented");
	//	return 0;
	}
	public ObjectInputStream.GetField readFields() {
		throw new RuntimeException("readFields() invoked - not implemented");
	//	return null;
	}
 


	/**	==========================================================================
		* TEST CODE
		* =========
		**/
	public static void main(String[] args) throws Exception {
		ObjIn in = new ObjIn();
		ObjOut out = new ObjOut();
		Object o;
		while ( (o = in.readObject()) != null) {	//loop for streamed XML
			if (DEBUG) {	//sep debug info from deserialization and serialization
				System.err.println();
				System.err.println("===============================================================================");
				System.err.println();
			}
			out.writeObject(o);
		}
	}
}
