/** (c) Brendan Macmillan 2001, licensed under GNU's GPL 2 */
/** (Also available for non-GPL, commercial use, for fee) */
/** Website: http://www.csse.monash.edu.au/~bren/JSX
/** Website: http://freshmeat.net/projects/jsx


/** ===========================================================================
	* ObjOut - (could be called XMLObjectOutputStream)
	*	======
	* Output/serialization.
	* There are many possible calls to the Serialization API; but they are
	* all callbacks, from within writeObject(), and so in a sense are invisible.
	**/

package JSX;
//package ObjOutDev;
//import JSX.*;	//use this, not package, if trully a "wrapper"
import java.io.*;	//Stream
import java.util.*;	//Vector eg

public class ObjOut extends ObjectOutputStream {
	final boolean DEBUG = false;
	//final boolean DEBUG = true;

	private XMLSerialize s;	//new, for thread-safe version
	public ObjOut(PrintWriter out) throws IOException {
		super();
		s = new XMLSerialize(out);
			//this.out = out; //now maintained within XMLSerialize
		s.setArg(new Object[] {this});	//call-back (best: all in one)
	}
	public ObjOut(OutputStreamWriter out, boolean autoFlush) throws IOException {
		this(new PrintWriter(out, autoFlush));
	}
	public ObjOut(boolean areAliasesUsed, OutputStreamWriter out) throws IOException {	//links to above
		this(out, true);	//autoFlush by defualt, nice for demo
		s.areAliasesUsed = areAliasesUsed;
	}
	public ObjOut(OutputStream out, boolean autoFlush) throws IOException {
		this(new OutputStreamWriter(out), autoFlush);	//pass on autoFlush switch
	}
		//to avoid conflict of sig with above.  Need to rationalize this con API!
	public ObjOut(boolean areAliasesUsed, OutputStream out) throws IOException {	//links to above...
		this(out, true);	//autoFlush by default
		s.areAliasesUsed = areAliasesUsed;
	}
	public ObjOut(OutputStream out) throws IOException {	//links to above...
		this(out, true);	//autoFlush by default
	}
	//private boolean areAliasesUsed=true; //default "on"; set in constructor


	//protected ObjOut() throws IOException, SecurityException {
	public ObjOut(boolean areAliasesUsed) throws IOException, SecurityException {
		this(System.out);		//NB: of class "OutputStream"
		s.areAliasesUsed = areAliasesUsed;
	}
	public ObjOut() throws IOException, SecurityException {
		this(System.out);		//NB: of class "OutputStream"
	}

/* ObjectOutputStream API */
	public void reset() { s.reset(); }
	public void flush() { s.flush(); }
	public void close() { s.close(); }

	protected void writeObjectOverride(Object obj) throws IOException {
		//s.serialize(obj);
		if (DEBUG) System.err.println("writeObject: " +obj);
		s.serialize(obj);	//fine; wrapper APIs must dup a little
	}
		//NB: *must* override, or OOS's version is called
	public void defaultWriteObject() throws IOException {
		if (DEBUG) System.err.println("defaultWriteObject() - need to pass in");
		s.defaultWriteObject();	//just "return" a flag
	}
	public void write(byte[] b) throws IOException {
		writeObject(b);	//handle it as an object (why not?)
	}
	public void writeInt(int data) throws IOException {
		s.writePrim(data+"");
	}
	public void writeFloat(float data) throws IOException {
		s.writePrim(data+"");
	}
	public void writeBoolean(boolean data) throws IOException {
		s.writePrim(data+"");
	}
	public void writeByte(int data) throws IOException {
		s.writePrim(data+"");
	}
	public void writeBytes(String data) throws IOException {
		System.err.println("Should this be written as an object (so alias is used?)");
		writeObject(data);	//handle it as an object (why not?)
	}
	public void writeChar(int data) throws IOException {	//"int" matches OOS sig
		s.writePrim(data+"");
	}
	public void writeChars(String data) throws IOException {
		System.err.println("writeChars() is not properly implemented");
		System.err.println("Should this be written as an object (so alias is used?)");
		writeObject(data);	//handle it as an object (why not?)
	}
	public void writeDouble(double data) throws IOException {
		s.writePrim(data+"");
	}
	//Float - see above
	//Int - see above
		//writeLong() is thanks Andy Thomas (who found Date didn't work).
		//He's the first successful JSX Challenger. Wed, 10 Jan 2001 22:33:57 +0000
		//James Skinnell found it, too (through java.sql.Timestamp not working).
		//He's the second successful JSX Challenger. Thu Jan 11 08:06:28 2001
	public void writeLong(long data) throws IOException {
		s.writePrim(data+"");
	}
	public void writeShort(int data) throws IOException {	//"int" matches OOS
		s.writePrim(data+"");
	}
		/**
		 	* writeUTF(str)
			* ------------
			* writeUTF specifies the data format, which is not relevant to us, as
			* we use an XML format.  However, it does have the side-effect of
			* treating a String 100% as a primitive, in the sense that there is
			* no aliasing (confirmed with Java's own Serialization: "java TestSuite e"
			**/
	public void writeUTF(String data) throws IOException {
		System.err.println("writePrim() is correct, not writeObject()");
		s.writePrim(data+"");
/*
		writeObject(data);	//handle it as an object (why not?)
*/
	}

	public static void main(String[] args) throws Exception {
		ObjOut out = new ObjOut();
		Object[] t = new Object[2];
		t[1] = new HashMap();
		t[0] = new ArrayList();
		out.writeObject(t);
		//out.flush();		//autoflush by default
//		new ObjOut(new FileWriter(new File("egout.xml"))).writeObject(t);
	//NB: this is inefficient, because we do the same thing twice, but it's
	//just an example.
	}
}
