/** (c) Brendan Macmillan 2001, licensed under GNU's GPL 2 */
/** (Also available for non-GPL, commercial use, for fee) */
/** Website: http://www.csse.monash.edu.au/~bren/JSX
/** Website: http://freshmeat.net/projects/jsx


/** ===========================================================================
  * XMLDeserialize (JSX0.8.8.5).
	* ==============
	*
	* Creates the object described by an XML document.

	* Note that a basic limitation of Java is that an object cannot be created
	* unless the JVM has access to the class file defining it.  That is, Java
	* cannot *define* classes at runtime; it can only instantiate pre-defined
	* classes.
	* However, a class loader could be written to "read" in class files which
	* do not actually exist; thus giving Java the ability to define new
	* classes dynamically.  See also JavaClass (link on JSX homepage).

	* Please email me: for help, to suggest features, or to report bugs.
		bren@csse.monash.edu.au  <Brendan Macmillan>

	* ISSUES:
	*		What about: subclassing wrappers or containers?
	*		What if root object is a wrapper?  Or a field is a wrapper?
	
	**/


package JSX;
import java.lang.reflect.*;
import java.io.*;
import java.util.*;	//for Hashtable

public class XMLDeserialize {

/** --------------------------------------------------------------------------
	* debug constants
	* ---------------
	**/
	private static final boolean DEBUG_CUSTOM = false;
	//private static final boolean DEBUG_CUSTOM = true;

	private static final boolean DEBUG = false;
	//private static final boolean DEBUG = true;

	private static final boolean TESTCIRC = false;
	//private static final boolean TESTCIRC = true;

	private static final boolean ALIASDEBUG = false;
	//private static final boolean ALIASDEBUG = true;

	private static final boolean INTERNAL_DEBUG = false;
	//private static final boolean INTERNAL_DEBUG = true;

	private static final boolean DEBUG_CLASS = false;
	//private static final boolean DEBUG_CLASS = true;

	private static final boolean FINAL_DEBUG = false;
	//private static final boolean FINAL_DEBUG = true;

/** --------------------------------------------------------------------------
	* Deserialization fields
	* ----------------------
	* NB: there are others scattered through the code, proximate to their use.
	**/
	private ParserXML p;

/** --------------------------------------------------------------------------
	* alias fields
	* ------------
	* Consider: extract as an object.  It has state, init, and API.
	* Consider: change name from 'aliasHash' to 'aliasTable'.
	**/
	private int aliasSerialNumber;	//NB: *not* static (cs serialization)
	private Hashtable aliasHash = new Hashtable();
		//private static final int ALIAS_INIT = -1;//so pre-increment in serializeObject
		//get INIT directly from XMLSerialize, so guaranteed to be synchronized.


	
	/**--------------------------------------------------------------------------
		* main
		* ----
		* Version information only; for test code, see JSXTest.java
		**/
	public static void main(String args[]) {
		System.err.println(XMLSerialize.VERSION);	
	}



/**==========================================================================
	* CONSTRUCTORS
	* ============
	**/
	/**--------------------------------------------------------------------------
		* XMLDeserialize(in) - core constructor
		* --------------
		* Core constructor (others call it)
		* init alias *only* here, or confuses re-enterant code (internal ser)
		**/
	public XMLDeserialize(Reader in) {	
		reset();	//whenever create, always reset alias table.
		p = new ParserXML(in);
	}
	//--------------------------------------------------------------------------
	/** XMLDeserialize()
		* --------------
		* Default to System.in
		**/
	public XMLDeserialize() {			//default to System.in
		this(new InputStreamReader(System.in));
	}



/**==========================================================================
	* METHODS
	* =======
	**/
	int invocationCount = 0;
	//--------------------------------------------------------------------------
	/** deserialize()
		* -----------
		* Entry point.
		* NB: invocationCount distinguisges between first time callers, and
		* calls from internal serialization.  This allows both old and new
		* API's to work.
		**/
	
	public Object deserialize() throws Exception {
		if (INTERNAL_DEBUG) System.err.println("RETRIEVE optional data object: "+objStoreIndex);
		if (invocationCount==0) {
			reset();
			invocationCount++;
			ParserXML.Tag tag = p.readTag();
			if (tag==null) return null;		//no more objects left
			if (!tag.start)
				throw new ParserXML.ExceptionXML("Expected start tag, got \"</" +tag.name+ ">\"");
			//Object o = createObject(tag.name, null);	//no parent
			Object o = createObject(tag, null);	//no parent //10 Apr 2001
			if (TESTCIRC)
				System.err.println("Count was: " + (aliasSerialNumber-XMLSerialize.ALIAS_INIT));
				if (ALIASDEBUG) System.err.println(aliasHash);
			invocationCount--;
			return o;	//separate statement into 2 lines so we can print the count
		}
		else {//ie. if from a customization
			return getObj();
		}
	}

	//NB: same as XMLSerialize
	public void reset() {
		if (invocationCount!=0)
			throw new Error("Deserialiize: Attempt to reset alias table mid-way.");
		initAlias();
	}
	public void close() {	p.close(); }



	/** =========================================================================
		* alias nascent-object
		* --------------------
		* NB: init, fields and API: extract into a class.
		* Should move this into a separate, pluggable class, so that
		* (1). the implementation can be changed easily
		* (2). a co-developer can work on it in isolation
		* (3). it can be recompiled separately - faster recompilation
		* (4). it can be reused elsewhere
		* (5). a different implementation can be plugged in (including a subclass)
		*
		* All are aspects of "divide and conquer" - the danger is to divide 
		* overzealously, in an inappropriate way, which counterproductively
		* increases complexity.  However, when a division is clear, it really
		* should be done.
		* The reluctance to do it is that the change may introduce bugs.  It is
		* work, and uncertainty, and fear of opening a can of worms - better to
		* let sleeping dogs lie.
		* And yet, because it has been used as it were a separate object, it
		* should not be too hard.  And if there are ripples in the force, it is
		* good to find them and tame them, rather than fear them.
		*
		* It would also be good to have an "alias" API - javadocs could produce
		* this automatically.  Makes it easier to work with.  Looks more
		* professional (to employers, licensees), and more welcoming to 
		* co-developers.   Reduces barriers to contribution.  Increases chances
		* of patches being offered.
		**/ 
	private void initAlias() {
		aliasSerialNumber = XMLSerialize.ALIAS_INIT;
		aliasHash.clear();
	}
	/**--------------------------------------------------------------------------
		* put Alias
		* --------
		*	inline = !serialized;
		* if (inline)
		*		//add to hash
		*	else
		*		//write as alias
		*	return (inline);		//to tell caller to serialize it inline!
		*		//else, caller leaves the ref writing up to this routine
		**/
	//private static boolean putAlias(Object o, String fieldName, boolean isStringAttr, String indent) {

	/**--------------------------------------------------------------------------
		* getAlias
		* --------
		* Much of this is duplicated code from addAttr().  This should be
		*	rationalized and refactored at some point.
		**/
	boolean aliasNameMissing;	//clumsy message passing
	private NamedObject getAlias(String flag, Object parent) throws ParserXML.ExceptionXML, Exception {
		aliasNameMissing = false;
		ParserXML.Attr attr;
		String fieldName = null;
		String aliasName = null;
		if (flag.equals(XMLSerialize.ALIAS_TAG_TOKEN)) {	//parse obj-name and name
			while ( !(attr=p.readAttr()).isEnd) {
				if (attr.name.equals(XMLSerialize.NAME_TOKEN)) {
					if (fieldName!=null) 
						throw new ParserXML.ExceptionXML("just one name field");
					fieldName = attr.value;
				}
				else if (attr.name.equals(XMLSerialize.ALIAS_ATTR_TOKEN)) {
					if (aliasName!=null)
						throw new ParserXML.ExceptionXML("just one alias name");
					aliasName = attr.value;		//NB: won't flag duplicates
				} else
						throw new ParserXML.ExceptionXML("unknown attribute: "+attr.name);
			}
 			Object me = aliasHash.get(aliasName);
			if (parent!=null) {	//ie, if not an orphan object (as in Vector, Array)
				if (fieldName==null)
					aliasNameMissing = true;	//need to defer it, so context is correct
				else {
					Field f = getAllField(parent.getClass(), fieldName);
					setFinal(f, parent, me);
					//f.set(parent, me);
				}
			}
				//System.err.println("alias " +aliasName+ ":");
				//System.err.println("me: " +me);
 			return new NamedObject(fieldName, me, !aliasNameMissing);
		}
		else
			return null;	//indicates that the tag was not an alias
	}

	private static void setFinal(Field f, Object parent, Object value)
throws Exception {
		if (FINAL_DEBUG) {
			System.err.println("setFinal");
			System.err.println(f);
			System.err.println(parent);
			System.err.println(value);
		}
		if (Modifier.isFinal(f.getModifiers())) {	//if final...
			//object and primitive need osf.  We simulate a non-private constructor:
					/*		ObjectStreamField(Field field) {
					*			this(field.getName(), field.getType());
					*			this.field = field;
					*		} */
			//How can this be achieved more simply?
			ObjectStreamField osf = new ObjectStreamField(f.getName(), f.getType());
			Field osfField = ObjectStreamField.class.getDeclaredField("field");
			osfField.setAccessible(true);
			osfField.set(osf, f);
			//The previous approach was:
				//ObjectStreamClass osc = ObjectStreamClass.lookup(parent.getClass());
				//System.err.println(parent.getClass());
				//System.err.println(osc);
				//System.err.println(f.getName());
				//ObjectStreamField osf = osc.getField(f.getName());
			if (FINAL_DEBUG) {
				System.err.println(osf);
				System.err.println("about to set...");
			}
			if (f.getType().isPrimitive()) {	//if prim...
				MagicClass.setPrimitiveFieldValues(parent, osf, value);
				if (FINAL_DEBUG) System.err.println("...set primitive final field!");
			} else { //if obj...
				MagicClass.setObjectFieldValue(parent, osf, f.getType(), value);
				if (FINAL_DEBUG) System.err.println("...set object!");
			}
		} else {	//if non-final:
			//do with reflection
			//Future: tigher integration with above (array based) is faster.
			f.set(parent, value);	//<--- this doesn't work
		}
		if (FINAL_DEBUG) System.err.println("finished setFinal");
	}

	/** ------------------------------------------------------------------------
		* putAlias
		* --------
		* Change to return the alias - this should be a "back-compatible
		* refactoring" for callers (tho all return statements need to be changed),
		* and it allows it to be stored, and used to later update it (for
		* readResolve()).
		**/
	private String putAlias(Object o) {
		if (o==null) return null;	//a flag: null not a valid alias - but...
			//better to throw an exception?
		++aliasSerialNumber;	//pre-inc (use this syntax to highlight it)
		String alias = aliasSerialNumber+"";
		aliasHash.put(alias, o);	//add (NB: inverse of serialize)
			//put() returns the previous key - it *shouldn't* have had one (ie null)
		if (TESTCIRC) System.err.println("inline " +aliasSerialNumber+ ":");
		return alias;
/*
		Object alias;	//will be String, but save the conversion
		if ( (alias = aliasHash.get(o))==null) {
			++aliasSerialNumber;	//pre-inc (use this syntax to highlight it)
			aliasHash.put(o, aliasSerialNumber+"");	//add (convert to String)
				//System.err.println(indent +"inline " +aliasSerialNumber+ ":");
			//Need to:
			//	indicate "inline" somehow.
			//	write as inline (same as now).
			return true;	//yes, should serialize the object inline
		}
		else
		{
			if (isStringAttr) {
				printAttr(fieldName + STRING_ALIAS_TOKEN, alias+"");
			}
			else {
					//System.err.println(indent+ "alias " +alias+ ":");
				printOpenTag(ALIAS_TAG_TOKEN, indent);
				if (fieldName!=null) printAttr(NAME_TOKEN, fieldName);	//nice to reuse
				printAttr(ALIAS_ATTR_TOKEN, alias+"");
				out.println("/>");		//empty tag - under control of caller
			}
			return false;	//no, don't serialize inline; we (or you?) write the alias
		}
*/
	}

	/** ------------------------------------------------------------------------
		* updateAlias
		* -----------
		* for readResolve() - easy to implement, because Hashtable already
		* allows for updates
		**/
	private String updateAlias(Object o, String alias) throws IOException {
		if (o==null) return null;	//a flag: null not a valid alias - but...
			//better to throw an exception?
		if (aliasHash.put(alias, o)==null)	//add (NB: inverse of serialize)
			throw new IOException("update called, but no existing alias");
			//this returns the previous key - it *should* have had one.
		return alias;
	}


	static final Class[] OIS_ARGS = {ObjectInputStream.class};
	public Object[] readObjectArglist;
	/**--------------------------------------------------------------------------
		* setArg()
		* -------
		* Consider: putting this in constructor; or caller in same package or same
		* class.
		**/
	public void setArg(Object[] argList) {
		readObjectArglist = argList;
	}
	Vector primStore = null; //Note: not yet set!  Consider: best place to define?
	int primStoreIndex = -1;	//fail fast
	/**--------------------------------------------------------------------------
		* getPrimString()
		* ---------------
		* Called by readInt(), readFloat(), etc to get the String that was
		* stored by addAttr
		**/
	public String getPrimString() {
		return (String) primStore.get(primStoreIndex++);
	}

	Vector objStore; //Consider: best place to define?
	int objStoreIndex;
	/**--------------------------------------------------------------------------
		* getObj()
		* --------
		* Called by readObject(), when it is a from a customization (not the
		* top-level call).  The Vector is populated by createObject.
		**/
	public Object getObj() {
		return objStore.get(objStoreIndex++);
	}

	/**--------------------------------------------------------------------------
		* NamedObject - message object (to return more than one thing)
		* -----------
		* Defn: Wraps an "object" and its (field)"name".
		* Used by: createObject. null, alias, array etc routines return it.
		*
		* Future: We return a message object.  Object creation is inefficient.
		* Reuse a static object - a global variable with cleaner semantics (usage).
		* That is, set global/static object fields, and return it.  Problem is that
		* not restricted to this use - others could still access it as global -
		* unless the orginator of it was in a separate class.  An advantage of
		* small modules.
		**/
	class NamedObject {
		String name;
		Object object;	//better to not use abbreviations
		boolean named;
		NamedObject(String name, Object object, boolean named) {
			this.name = name;
			this.object = object;
			this.named = named;
		}
	}


	/**--------------------------------------------------------------------------
		* createObject()
		* ------------
		* Returns the deserialized object (caller needs to cast at runtime)
		* The object created is "me".
		* USAGE: the open tag is assumed to have been just read
		* LOGIC: A new object is instantiated: "me".  Attr and elements are
		* added to me.  This new object is then added to its parent.  NB: that
		* the name for me (given by parents) is only discoverable by reading the
		* attr; which is done in "addAttr" (second arg is boolean for isArray)
		* NB: a null object may still have a name.
		* NOTE: tag problem for lengths - not actually a field.
		*
		* Issues are:
		*	 - custom data? Unnamed fields.
		*  - Strings - attr or tags? strings in Object fields.
		**/
	private Object createObject(ParserXML.Tag tag, Object parent)
			throws Exception {
		String elementName = tag.name;	//loosely coupled argument passing
		Object me;
		boolean nameMissing = false;	//named assumed to be present
		NamedObject namedObject;	//used in many sections below:

//NULL
		if (elementName.equals(XMLSerialize.NULL_TOKEN)) {
			namedObject = deserializeNull(parent); //code dup for array and alias
			me = namedObject.object;	//it will be null
			nameMissing = !namedObject.named;	//this is the info we need
			if (nameMissing) {	//only consider objStore, if name is missing...
				if (parent!=null) {	//if not an orphan, for all these...
					if (parent.getClass()!=Vector.class &&			//not vector
							parent.getClass()!=Hashtable.class &&		//not hashtable
				 			!parent.getClass().isArray()) {					//not array
						objStore.add(me);
					}
				}
			}
			//NOTE: a bug was fixed in this refactoring: before, such a string was
			//added to objStore, regardless of whether named or not.  However, this
			//wouldn't confuse any custom objects, because this extra one would just
			//be tacked on the end.
			return me;
		}

//ARRAY
		else if (elementName.startsWith(XMLSerialize.ARRAY_TOKEN)) {
			namedObject = createArray(elementName, parent);
			me = namedObject.object;
			nameMissing = !namedObject.named;	//this is the info we need
			if (nameMissing) {	//only consider objStore, if name is missing...
				if (parent!=null) {	//if not an orphan, for all these...
					if (parent!=null && //not root (MEANT to be "not orphan"?)
							parent.getClass()!=Hashtable.class &&
							parent.getClass()!=Vector.class &&
							!parent.getClass().isArray()) {
						objStore.add(me);	//old bug: no objStore exists.
					}
				}
			}
			//Only put in objStore if custom data. That is, if an "unnamed field":
			//parent is a classInstance and object has no name.
			return me;		//Limit interaction with rest of routine.
			//NOTE createArray() reads close tag, to completely parses the element.
			//validation of nested of array types is left to Java.
		}

//ALIAS
		else if ( (namedObject=getAlias(elementName, parent))!=null ) {
			me = namedObject.object;
			nameMissing = !namedObject.named; //if named, plugged inside already
			if (nameMissing) {	//only consider objStore, if name is missing...
				if (parent!=null) {	//if not an orphan, for all these...
					if (parent.getClass()!=Vector.class &&			//not vector
							parent.getClass()!=Hashtable.class &&		//not hashtable
			 				!parent.getClass().isArray()) {					//not array
							objStore.add(me);
					}
				}
			}
			return me;
		}
	
//STRING
		else if (elementName.equals("java.lang.String")) {
				//EXAMPLE:  Object eg = "a string";  //<- String as named object
				//un-named: element of Hash or Vect, array of Object(?), root

			namedObject = deserializeWrapper(tag, parent);
				//(1). sets the field, if name is null.
				//(2). putAlias()
			nameMissing = !namedObject.named; //if named, already plugged
			me = namedObject.object;
			if (parent!=null) {	//if not an orphan, for all these...
				if (nameMissing) {	//only consider objStore, if name is missing...
					if (parent.getClass()!=Vector.class &&			//not vector
						parent.getClass()!=Hashtable.class &&		//not hashtable
						!parent.getClass().isArray()) {						//not array
						objStore.add(me);	//NOTE: we store it as an Object, not prim!
					}
				}
			}
			return me;
		}

//CLASS
		else if (elementName.equals("java.lang.Class")) {
			if (DEBUG_CLASS)
				System.err.println("Got a Class tag");
			namedObject = deserializeClass(tag, parent);
				//(1). sets the field, if name is null.
				//(2). putAlias()
			nameMissing = !namedObject.named; //if named, already plugged
			me = namedObject.object;
			if (parent!=null) {	//if not an orphan, for all these...
				if (nameMissing) {	//only consider objStore, if name is missing...
					if (parent.getClass()!=Vector.class &&			//not vector
						parent.getClass()!=Hashtable.class &&		//not hashtable
						!parent.getClass().isArray()) {						//not array
						objStore.add(me);	//NOTE: we store it as an Object, not prim!
					}
				}
			}
			if (DEBUG_CLASS) {
				System.err.println(">>>>start>>>>>");
				System.err.println(me);
				System.err.println( ((Class)me) .newInstance());
				new JSX.ObjOut(System.err).writeObject( ((Class)me) .newInstance());
				System.err.println("<<<<end<<<<");
			}
			return me;
		}

//VECTOR
		else if (elementName.equals("java.util.Vector")) {
			Class clazz = Class.forName(elementName); //2 lines, for debugging
			me = clazz.newInstance();
			ParserXML.Attr name = null;
			name = addAttr(me, false);//gets NAME_TOKEN - should be no primitive data
	 		nameMissing = name.nameMissing;
			if (parent!=null) { //can't plug if no parent (vect, hash, array do sep)
				//(1). BUFFER (CUSTOM DATA)
	 			if (nameMissing) {//no name
					if (parent.getClass()!=Vector.class &&			//not vector
							parent.getClass()!=Hashtable.class &&		//not hashtable
							!parent.getClass().isArray()) {						//not array
						objStore.add(me);	//store custom *Object* in buffer
					}
				}
				else {	//if named...
				//(2). INTO CLASS AS FIELD
					Field f = getAllField(parent.getClass(), name.value);
					//f.set(parent, me);
					setFinal(f, parent, me);
				}
			}
			putAlias(me);	//to sync with ser: do before putAlias for elements
	 		if (!name.emptyTag) {	//<java.util.Vector/> means no elements to add
				addVectorElements((Vector) me);
			}
			return me;	//note: could return null, if already plugged with name?
		}

//HASHTABLE
		else if (elementName.equals("java.util.Hashtable")) {
			Class clazz = Class.forName(elementName); //2 lines, for debugging
			me = clazz.newInstance();
			ParserXML.Attr name = null;
			name = addAttr(me, false);//gets NAME_TOKEN - should be no primitive data
	 		nameMissing = name.nameMissing;
			if (parent!=null) { //can't plug if no parent (vect, hash, array do sep)
				//(1). BUFFER (CUSTOM DATA)
	 			if (nameMissing) {//no name
					if (parent.getClass()!=Vector.class &&			//not vector
							parent.getClass()!=Hashtable.class &&		//not hashtable
							!parent.getClass().isArray()) {						//not array
						objStore.add(me);	//store custom *Object* in buffer
					}
				}
				else {	//if named...
				//(2). INTO CLASS AS FIELD
					Field f = getAllField(parent.getClass(), name.value);
					setFinal(f, parent, me);
					//f.set(parent, me);
				}
			}
			putAlias(me);	//to sync with ser: do before putAlias for elements
	 		if (!name.emptyTag) {	//<java.util.Hashtable/> means no elements to add
				addHashtableElements((Hashtable) me);
			}
			return me;	//note: could return null, if already plugged with name?
		}

//CLASSINSTANCE (the entire remainder of routine)
// ---just written: 2 Aug 2001---
//proposed change - just a switch in order, and move some to default() method.
//	- create
//	- foreach class, call readObject
//	- else default
//		- default is addAttr, plug, addTag
//	NB: need addAttr for name, to decide how to plug.
//	NB: always call default *exactly* once
//	NB: maybe problems with stores (for custom data)
//	NB: this fix only works if only one class in the inheritance path
//		has a readObject() that relies on the defaultReadObject() order.
//		- and fortunatel, this does seem to be the case.
		else {
			Class clazz = Class.forName(elementName);	//note: code dup with above
			me = MagicClass.newInstance(clazz);	//use anon
			if (DEBUG) System.err.println("Made class \"" +elementName+ "\"");
				//Constructor con = clazz.getConstructor(new Class[] {});	//no-arg con
				//con.setAccessible(true);
				//me	= con.newInstance(new Object[] {});	//no-arg again
		
	//6 Aug 2001 - need to store present alias *if* has readResolve.
			String alias = putAlias(me);	//6 Aug 2001
			//putAlias(me);	//NOTE: do before any Strings in addAttr()
				//Be careful of this complexity!
				//To make consistent, may need to change the above
			Vector localStackPrimStore = primStore;	//push old (see Explanatory memo)
			int localStackPrimStoreIndex = primStoreIndex; 
			primStore = new Vector(); //new context - NOTE: need for addAttr
			primStoreIndex = 0;
				//DESIGN DECISION: custom data Strings never in attributes!

			ParserXML.Attr name = null;
			name = addAttr(me, false);	//adds in all attr and gets NAME_TOKEN
	 		nameMissing = name.nameMissing;
			if (DEBUG && name!=null) System.err.println("Name: "+name);
			if (DEBUG) System.err.println("nameMissing: "+nameMissing);

		 /* Explanatory memorandum for custom data:
			*	--------------------------------------
			*	Custom data only occurs in classInstances (not null, alias, String,
			*	hash or vect).  It has two phases: (1). Store custom data in buffers
			*	(all attrs and contained, nested tags).  (2). Callback the objects's
			*	readObject() method, drawing from the buffers, via calling readObject
			*	within it readInt(), readLong() etc.  The buffer context needs to be
			*	stacked, because contained tags (field or custom data objects) may
			*	themselves contain custom data.  Both fields and custom data object are
			*	created the same; the difference is that custom data is stored in a
			*	stacked buffer.  This stack is implemented with the local var
			*	stackframe (CONSIDER: would an explicit stack be clearer?).
			*
			* In effect, each classinstance has two potential kinds of data - in
			* fields, and in buffers.  The buffer is therefore like a kind of series
			* of positional field, or typeless array.  This provides another view of
			* the need for stacking: each object has its own buffer, just as it has
			* its own field.
			*
			* Attributes are stored into their parent buffer when we are in their
			* "parent" object.  This is because attributes can never be parents -
			* there is no possibility of nesting (tho we still need to stack).  In
			* contrast, objects are stored into the parent buffer when we are in the
			* "child" object.  This is because we always change the context for
			* objects.
			*
			* This difference is caused by not changing context for attributes - it
			* might be good to regularise it by representing the present object
			* context as a "parent" when doing the attributes.
			*
			* Note: custom data String objects are never stored in attributes.  This
			* avoids the problem of determining that it is a String (and so needs to
			* create an alias, and be put in objStore, not primStore).  The type of
			* fields can be determined from the class; but not so for custom data.
			* Although the position does ultimately determine how it is read, this is
			* deferred (and so can cause problems for aliasing), and the attr/tag
			* split (seems) to not preserve the ordering - unless we know into which
			* to put the String beforehand (begging the question)!  Easier to simply
			* put it in a tag!!  Previous design comments:
			*
			* >This order is in fact lost already - it was lost when the Strings were
			* >included as Attr, instead of as objects... the simplest thing may well
			* >be to represetn *every* string as a wrapper... or, at least, to include
			* >every customized string as a wrapper...
			*
			* Note that we still need to stack attributes, even though they
			* themselves don't nest, since the object containing them will have
			* invocation of its readObject() method deferred until after any
			* contained tags (objects) are also processed, which need their own
			* "buffer field" for any custom data.
			*
			*/
	
			if (INTERNAL_DEBUG) {
				if (primStore!=null) {
					for (Enumeration keys=primStore.elements(); keys.hasMoreElements();) {
						Object a = keys.nextElement();
						System.err.println("In primStore: " +a+ " -- "
					 	+ a.getClass() + " -- " +a.hashCode());
					}
				}
			}

//CHILDREN TAGS; recursion, and a new context for buffering custom data
		 /*
			* A new custom data context is created, and then the children are done.
			* Each custom data child is buffered into this context - all siblings
			* into the same one.  This buffering is done in this routine
			* (createObject), with objStore.add().  Incidentally, they nest, so
			* memory usage is (aveObjectSize*depth).
			*/
			Vector localStackObjStore = objStore; //save locally, so recurse
			objStore = new Vector();	//this will get all the enclosed tags of "me"
			int localStackObjStoreIndex = objStoreIndex;	//also save it.
			objStoreIndex = 0;

			Object localMeStore = meStore; //Push
			meStore = me;
			boolean localEmptyTagStore = emptyTagStore; //Push
			emptyTagStore = name.emptyTag;
//addTags *is* the default!! -- now called from ObjIn!
//		if (!name.emptyTag) {	//ie if has elements (not <xxxx/>)
//			addTags(me);	//CENTRAL RECURSIVE dispatch: me am become the parent.
//		}

//CALLBACK object's readObject()
		 /*
			* The defaultReadObject() problem
			* -------------------------------
			* All contained tags are read; now invoke readObject() to get them.
			* Strictly, any fields should only be set if defaultReadObject() is
			* called.  But JSX doesn't work like this.  JSX sets them all if they are
			* present in the XML, before readObject() is even called.
			*
			* This rarely seems to be a problem - it would only be so if a class
			* *depended* on the recorded values not being set.  Usually, such a class
			* wouldn't write them in the first place - that is, it wouldn't call
			* defaultWriteObject() in writeObject(), and so the fields wouldn't be
			* present.  However, it might be relied upon in evolution of classes.
			*
			* A solution to this would be to not set the fields until after
			* readObject() is invoked; similar to as is presently done for the
			* custom data.  This means processing all fields twice - once to store,
			* and once to set into field.  Although it might also seem to create
			* a complete DOM tree replica of the object graph, it is not all in memory
			* at once, but only the path to the present object (on the stack).
			* (This does trace out an entire DOM tree, just not all at once).

			* Efficiency ("Don't" and "Not yet") - needn't do this when:
			*  - the parent is an Arrays, Vect or Hash.
			*  - there is no readObject() method (and so all fields are read)

			* The custom data Strings as attributes problem
			* ---------------------------------------------
			* There is a separate problem for custom data Strings in attributes:
			* There is no markup to indicate the type of attributes, so that we
			* don't know whether they are Strings or primitives, and therefore
			* don't know what to do with them.  For fields, we can derive the type
			* via Java reflection, based on the name.  But for custom data, we
			* don't know until the readObject() requests it.  However, for this to
			* work, we would need to have the custom data all in one buffer, rather
			* than split in two.
			*
			* [The difficulty is to do with splitting it into object/primitive, which
			* loses the relative ordering between them, before we can decide whether
			* it is an object or primitive!  Catch 22.  Begging the question.  This
			* information is in fact lost in the serialization into primitive vs
			* object - that's the split, not the buffering.  I'm a still a bit
			* confused about *exactly* what is wrong with this, which is why it's
			* not quite expressed right.  However, I'm convinced that it is an
			* insurmountable problem, due to the attr/object split.  The simple
			* resolution is just to always put custom data Strings into tags.]
			*
			* A solution to the buffering problem is to just do all the custom data
			* in one section, and so not need to buffer.  The readObject() could read
			* this directly, skipping named data.  This would require putting the
			* custom data at the end (so no buffering needed), or putting it into a
			* new "<custom>" tag - which would also need to go at the end. However,
			* the attr/tag split problem for Strings still remains.  This could be
			* solved by putting all custom data into tags - though we would need
			* someway to distinguish between primitives and their object wrappers.
			* Ie between Byte and byte.  Of course, we simply introduce <byte
			* valueOf=""/> tags and so on.

			* NOTE: in practice, a class not calling defaultReadObject() also should
			* not call defaultWriteObject().  If so, the fields are *not* serialized,
			* and so the defaultReadObject() has no effect, in practice.  This is not
			* the proper solution, but it may well work in all cases in practice.
			* Something that *does* need to be fixed for correct implementation of
			* the Java Serialization API.

			* SUMMARY  (rough, first draft)
			* -------
			* (1). Buffering of fields (as well as custom data) is *not* too memory
			* intensive - just a little stack overhead.  It will be *slightly* less
			* efficient.  It will solve the defaultReadObject() problem (though which
			* should never occur in practice, anyway).  NOT PLANNING TO DO YET.
			* (2). The attr/tag custom data String problem needs a dramatic change
			* to fix properly.  Since there seems no disadvantage to simply
			* serializing in a tag, that is what has been done.
			* (3). the custom data represents the positional fields of each object.
			* (4). fields are created and set as they are read.
			* (5). custom data is fully created and stored as it is read - then
			* processed after *all* tags have been read.
			* (6). tags (objects) are stored into a buffer belonging to the parent
			* object.
			* (7). attr (primitives) are stored into a buffer belonging to the
			* present object.
			*/

//CALLBACK OF readObject() for CUSTOM DATA
			boolean localDefaultCalledStore = defaultCalledStore; //Push
			defaultCalledStore = false;
			Method m;
			Vector classes = new Vector();	//NB: Mirror order - reverse.
			Class[] superClasses=XMLSerialize.getReversedSuperClasses(me.getClass());
			for (int j=0; j<superClasses.length; j++) {  //traverse root to leaf
				Class c = superClasses[j];
				if ((m=XMLSerialize.getDeclaredMethod(c, "readObject", OIS_ARGS,
						Modifier.PRIVATE, Modifier.STATIC)) != null) {	//NB: ?clazz->c
					if (INTERNAL_DEBUG) System.err.print("\t--JSX deserialize--");
					try {
						m.invoke(me, readObjectArglist);	//might as well do them all!
						if (INTERNAL_DEBUG)
							System.err.println("\tinvoke "+m+" on "+me+" of " +me.getClass());
					} catch (InvocationTargetException e) {
						System.err.println("JSX InvocationTargetException:");
						System.err.println("Object which is a: " +me.getClass());
						e.printStackTrace();	//should we just rethrow this?
					} catch (Exception e) {
						System.err.println("not an InvocationTargetException");
						e.printStackTrace();
					}
				} else {
					defaultReadObject(); //call if no readObject() declared
						//potential BUG: will JSX work if the readObject doesn't call
						//defaultReadObject?  Doesn't there need to be some check, so
						//that it does get called, if it hasn't been already?
						//This is like the other side of ensuring it is only called
						//once.
				}
			}
				//how is the best way (most modular) for readResolve to operate?
			me = readResolve(me, alias);	//defaults to me, if no readResolve

			defaultCalledStore = localDefaultCalledStore; //pop
			//for defaultReadObject
			meStore = localMeStore; //pop
			emptyTagStore = localEmptyTagStore; //pop

			//for Objects (tags)
			objStoreIndex = localStackObjStoreIndex;	//restore
			objStore = localStackObjStore;//restore objects for previous level

			//for Primitives (attr)
			primStoreIndex = localStackPrimStoreIndex;	//restore
			primStore = localStackPrimStore;//restore primitives for previous level


//PLUG: CUSTOM DATA->BUFFER; FIELD->PARENT
//There is no real reason that this cannot be at the end.  It is even an
//appropriate place for it to be - just before it is returned, which is
//the alternate mechanism.
//--Note that this section uses variables that are set from above--
	//6 Aug 2001 - QUESTION: can we move both set and add to after readObject?
	//They both seem to only relate to the parent, which is invariant
	//across the readObject invocation - great if could confirm this.
	//DEPENDS ON: me, parent, nameMissing, nameValue, a nested objStore.
		//addTags is done by the call to defaultReadObject(), which the object's
		//readObject() needs to invoke to setup the custom data for reading.
		//Basically: the "parent" context (parent, objStore) is set up around
		//the readObject/defaultReadObject calls... ah! The naming shows that it is
		//the central thing - even when not explicitly called, it still is called.
		//Our confusion is mainly caused by addAttr, which is because we mix
		//info *about* object (name, class, etc) with *content* of the object 
		//itself - ie primitives and strings.  This is conceptually inconsistent.
			if (parent!=null) { //can't plug if no parent (vect, hash, array do sep)
				//(1). BUFFER (CUSTOM DATA)
	 			if (nameMissing) {//no name
					if (parent.getClass()!=Vector.class &&			//not vector
							parent.getClass()!=Hashtable.class &&		//not hashtable
							!parent.getClass().isArray()) {						//not array
						objStore.add(me);	//store custom *Object* in buffer
						if (INTERNAL_DEBUG)
							System.err.println("custom obj["+objStoreIndex+"],"+objStore);
					}
				}
				//(2). INTO CLASS AS FIELD
				else {
					if (DEBUG_CUSTOM) {
						System.err.println("getting field '"+name.value+"'");
					}
					Field f = getAllField(parent.getClass(), name.value);
					setFinal(f, parent, me);
					//f.set(parent, me);
				}
			}

			return me;
		}
	}

/** ---------------------------------------------------------------------------
	* readResolve
	* -----------
	* Reflectively invoke the present objects's readResolve().
	* Return value can be: an object or an exception.  We are using an 
	* Exception to flag the result, instead of trying to encode it in the
	* return value itself (by wrapping it, for example - which is sort of how
	* Exceptions work, anyway.
	* QUESTION: If we checked for it separately, we can choose whether to cache
	* or not...
	* "parameterless" = no-arg
	**/
	Object readResolve(Object me, String alias) throws ObjectStreamException {
		try {
			Method readResolveMethod = me.getClass().getDeclaredMethod("readResolve", new Class[]{});
			readResolveMethod.setAccessible(true);
			Object newme = readResolveMethod.invoke(me, new Object[]{}); //no arg
			updateAlias(newme, alias);	//only need to update if it really is new.
			return newme;
		} catch (NoSuchMethodException e) {
			//Not an error - just means that it doesn't implement readResolve
			return me;	//if no readResolve - the object is simply unchanged!
		} catch (InvocationTargetException e) {
			System.err.println("JSX InvocationTargetException:");
			//e.printStackTrace();	//not sure how this should be handled...
			//is this the best way to handle it?  What does Serialization do?
			//Probably best to throw whatever is thrown.
			try {
				throw (ObjectStreamException) e.getTargetException(); //uncork it
				//contract of readResolve guarantees that it only throws this class
				//I should check that it *does* throw only this kind...
			} catch (ClassCastException f) {
				throw new Error("readResolve() of "+me.getClass()+ " threw an '"+e.getTargetException()+"'. It should only be of type ObjectStreamException");
			}
		} catch (Exception e) {
			System.err.println("not an InvocationTargetException");
			e.printStackTrace();
		}
		return null;	//to signal some error.
	}

	static boolean defaultCalledStore;	//so only called once per object
//storage for cross invocation communication
	static Object meStore;
	static boolean emptyTagStore;
//Called from ObjIn and also from here
	void defaultReadObject() {
		try {
			if (!defaultCalledStore) {	//only do it once, until reset
				defaultCalledStore = true;
				if (!emptyTagStore) {
					addTags(meStore);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}



	/**--------------------------------------------------------------------------
		* deserializeNull()
		* -----------------
		* Called from createObject(), to separate out the parsing code.
		**/
	private NamedObject deserializeNull(Object parent) throws Exception {
		String name = null;
		Object me = null;
		boolean named = false;
		ParserXML.Attr firstAttr = addAttr(me, false);	//gets NAME_TOKEN
		if (!firstAttr.nameMissing) { //see dup in createArray
			named = true;
			name = firstAttr.value; //NB: name of field is *value* of attr
		}
		if (!firstAttr.emptyTag)	//if not <null/>
			addTags(null);	//then start & end: <null></null> - read the end one.

	//again, code dup from createArray
		if (parent!=null) {
			if (named) {
				if (name==null) {	//should never happen
					throw new Exception("name: No " +XMLSerialize.NAME_TOKEN+ " attribute found for "+parent.getClass());
				}
				Field f = getAllField(parent.getClass(), name);
				setFinal(f, parent, me);
				//f.set(parent, me);
 				return new NamedObject(name, me, named);
			} else { //!named -> (internal: parent, but no name)
 				return new NamedObject(name, me, named);
			}
		} else {	//parent==null -> (root, array, hash, vect)
 			return new NamedObject(name, me, named);
		}
	}




	/**--------------------------------------------------------------------------t
		* getAllField()
		* -----------
		* Named in accord with the JSX serialization method "getAllFields()".
		* returns private and superclass fields.
		*  NOTE: could subclass Field instead.  More natural, as we are really
		*  extending its functionality.  However, it is also convenient to keep
		*  it in the one class.  And Field is final, anyway.  ;-)

		* DANGER: it could find a field from a superclass that customises its
		* own serialization.  This shouldn't happen. It really should check for
		* the readObject() method, and if present, disregard that declared class.
		*
		* We need to parse any class-qualified fields, and we can get these
		* directly from the named class - some unexpected efficiency.
		**/
	private	Field getAllField(Class c, String fieldName) throws NoSuchFieldException, SecurityException {
		Field f = null;
		if ((f=getShadowedField(fieldName))!=null) {	//if class-qualified
			f.setAccessible(true);		//so can see private fields (Java 1.2)
			return f; //then just grab it
		}
			//else continue on
		Class initc = c;	//store it for error
		do {
			try {
				f = c.getDeclaredField(fieldName);
			} catch (NoSuchFieldException e) {	//do nothing; it's OK
			} catch (Exception e) {
				System.err.println(e +": No such field '"+ fieldName +"' in "+ c +" (superclass of " +initc);
			}
		} while ( f==null && (c = c.getSuperclass()) != Object.class);
		if (f==null)
			throw new NoSuchFieldException("JSX can't find the field "
			+((fieldName==null)?"<null>":"'"+fieldName+"'")
			+ ".  Not in " +initc+ ", or private, or in superclasses\n"
			+ "Could it be that the class lacks a readObject() method?");
		f.setAccessible(true);		//so can see private fields (Java 1.2)
		return f;
	}

	static Field getShadowedField(String s) throws NoSuchFieldException {
		int dotpos = s.lastIndexOf('.');
		if (dotpos==-1) return null;	//signal: couldn't find (clarity?)
		String classQual = s.substring(0, dotpos);	//front
		String fieldName = s.substring(dotpos+1);	//end (skip the dot)
		Class clazz = null;
		try {
			clazz = Class.forName(classQual);
		} catch (ClassNotFoundException e) {
			System.err.println("The class-qualification of this shadowed field was not found: "+ s);
			e.printStackTrace();
		}
		return clazz.getDeclaredField(fieldName);
	}


/**==========================================================================**/
/**======================= ARRAY SECTION ====================================**/
/**==========================================================================**/

	/**--------------------------------------------------------------------------
		* createArray
		* -----------
		* Create one Array object.
		* Need to parse the whole tag name to know the component type,
		* 	eg: <ArrayOf-ArrayOf-int ..../>
		* that is: (ArrayOf-)*Type
		* 	(actually use a constant ARRAY_TOKEN = "ArrayOf-")
		* The mapping is:
		* 	(ArrayOf-)*Type -> Type([])*

		* It counts the ARRAY_TOKENS, extracts the type, then:
		*  Array.new_Instance(Class.getClass(baseType), new int[count])
		* NB: we avoid the complex disambiguation constraint on tok, by name choice

		* Complexity/Issues:
		*	 - Array of objects (nested tags)
		*	 - Array of primitives (attrs)
		*  - named: a field (parent is fields of a classInstance)
		*  - parent: only if a field or internal
		*  - unnamed: root, hash, vect, or array, internal
		*  - wrinkle 1: if parent and unnamed, then internal
		*  - wrinkle 2: if primitive, name attr can be before or after length attr
	  * NB: Internals are read in before we *know* they are internal, and they
	  * can *only* occur in classInstances, so they will have a parent.

		* Refactor for loose-coupled symbolic bindings
		* --------
		* Move all returned values to explicitly named variables for clarity
		*  - less efficient, but Knuth's first law of optimization is "don't".
		**/
	private NamedObject createArray(String tagName, Object parent) throws Exception {
		//Loose-coupling: move returned values to named variables
		int length = -1; //how many components of array
		String name = null;	//field name of array (if parent is a classInstance)
			//NB: Name may be in firstAttr or secondAttr, for arrays of primitives
		boolean named = false; //redundant clarity (named==(name!=null))
			//named==true: if a *field* (of a classInstance)
			//named==false: if *internal* info (of a classInstance),
			//		--or-- if *root* or *element* (of array, hash or vector).
		boolean emptyTag = false; //true=="<tag/>", false=="<tag>...</tag>"

		ParserXML.Attr firstAttr = addAttr(null, true); //object=null, isArray=true
		length = firstAttr.length;
		if (!firstAttr.nameMissing) { //if present, get name
			named = true;
			name = firstAttr.value; //NB: name of field is *value* of attr
		}
		emptyTag = firstAttr.emptyTag;

		//If an array of primitives, call addAttr twice:
			//(1). addAttr, to get length (and name, if present at start)
			//(3). create array
			//(2). addAttr, to get elements/attributes (and name, if present)
			//	NB: need length to create array
			//	NB: don't read elements til array created (could buffer in a Vector)
			//	NB: store name til after elements (attributes) are read

			//THUS: in addAttr (for array of primitives):
			// (1). addAttr to return after seeing "length"; and
			// (2). to call addAttr a second time, treating attr as indices.

			//THUS: in addAttr (for array of objects):
			// (1). reads the whole tag, including the ">" at the end of the tag.
		int dim = 0;
		int index=0;
		String tok = XMLSerialize.ARRAY_TOKEN;
		int skip = tok.length();
		//(1). take off the leading "ArrayOf-"'s (by incrementing an index for eff)
		while (tagName.startsWith(tok, index)) { //could be > eff if assume tok qual
			dim++;														//and just check first char...
			index+=skip;
		}
		if (DEBUG) System.err.println("dim= " +dim);
		String type = tagName.substring(index);	//get the type
		if (DEBUG) System.err.println("type= " +type);
		Object me = null;
		try {
			int[] dimsArray = new int[dim];
			dimsArray[0] = length;		//NOTE use of length parsed from attributes
			Class comp;
			//shockingly inefficient!  This is probably why they used the 'L'
			if (type.equals("int")) comp = int.class;
			else if (type.equals("double")) comp = double.class;
			else if (type.equals("boolean")) comp = boolean.class;
			else if (type.equals("char")) comp = char.class;
			else if (type.equals("byte")) comp = byte.class;
			else if (type.equals("short")) comp = short.class;
			else if (type.equals("long")) comp = long.class;
			else if (type.equals("float")) comp = float.class;
			else comp = Class.forName(type);
			me = Array.newInstance(comp, dimsArray);
			putAlias(me);
		} catch (Exception e) {
			System.err.println("Illegal type as Array base component, it seems; "+e);
		}
		if (DEBUG) System.err.println("type= " +me.getClass().getName());
		if (DEBUG) System.err.println("len= " +Array.getLength(me));


		ParserXML.Tag tag;
		ParserXML.Attr secondAttr = null;
		//If array of primitives...
		if (dim==1 && me.getClass().getComponentType().isPrimitive()) {
			secondAttr = addAttr(me, true); //object=me, isArray=true
				//addAttr now reads primitive attrs into array
			length = -1;	//shouldn't be used again (just did it in addAttr)
			if (!secondAttr.nameMissing) { //was name mixed in with attrs?
				if (named) throw new Exception("two names: "
					+"'"+firstAttr.value+"' and '"+ secondAttr.value+"' "
					+"in "+parent.getClass());
				named = true;
				name = secondAttr.value;	//NB: name of field is *value* of attr
			}
			emptyTag = secondAttr.emptyTag;
			//(1). above allows "obj-name" to be before or after elements.
			//(2). "length" must always preceed the elements (alt: vector buffer)
			//(3). firstAttr/secondAttr seems only to be needed for emptyTag.
			if (DEBUG) System.err.println("after setting primitive array, saw: " + secondAttr);
		}
 		//If array of objects...
		else {
			//do the contained objects (as does Vector) - it dispatches in a loop
			//NOTE  the recursion runs through create object, which tests isArray,
				//so as to automatically bottom out. AND WHAT A HACK IT IS!
			ParserXML.Attr end = p.readAttr(); //slurp up the left-over '>' after length read.
			if (DEBUG) System.err.println("after starting reference array, saw: " + end);
				//NB: this implies that the name (if present) *must* precede length
			for (int i=0; i<length; i++) {
				//System.err.println("i = "+i);
				Object obj;
		 		tag=p.readTag();
				if (tag==null) throw new Exception("Premature EOF");
				if (!tag.start)
					throw new Exception("Expected "+length+" elements in array; read only"+i);
				if (tag.name.equals("java.lang.String")	//bad: doubles up
						|| tag.name.equals("java.lang.Integer")
						|| tag.name.equals("java.lang.Double")
						|| tag.name.equals("java.lang.Boolean")
						|| tag.name.equals("java.lang.Character")
						|| tag.name.equals("java.lang.Byte")
						|| tag.name.equals("java.lang.Short")
						|| tag.name.equals("java.lang.Long")
						|| tag.name.equals("java.lang.Float")
					)
					obj = deserializeWrapper(tag, null).object;
				else
					//obj = createObject(tag.name, null);	//null=orphan object
					obj = createObject(tag, null);	//null=orphan object //30Apr2001
				if (DEBUG) System.err.println("Adding {"+obj+"} to Array");
				Array.set(me, i, obj);
			}	//for loop
		}	//tags (for refernce base type - array or class)

		if (!emptyTag) {		//if not <..../>, check for close tag
			tag=p.readTag();
			if (tag==null) throw new Exception("Premature EOF");
			if (tag.start)
				throw new Exception("End of array");
			if (!tag.name.equals(tagName))	//check for matching close tag
				throw new ParserXML.ExceptionXML(
					"</"+tagName+">","</"+tag.name+">");
		}
//parent is null (this is an "orphan") for:
	//root, arrays, hash and vect
	//NB: Internals are read in before we *know* they are internal, and they
	//can *only* occur in classInstances, so they will have a parent.

		if (parent!=null) {
			if (named) {
				if (name==null) {	//should never happen
					throw new Exception("name: No " +XMLSerialize.NAME_TOKEN+ " attribute found for "+parent.getClass());
				}
				//if name -> (plug & return null); if no name (return me).
				//For primitives, call addAttr() twice - initialAttr is first one.
				//26May - now initialAttr is the initial value (whether called once
				//or twice.
				Field f = getAllField(parent.getClass(), name);
				setFinal(f, parent, me);
				//f.set(parent, me);
					//return null;	//a signal: already been dealt with...
				//gets here if: has parent, and has attr.value, and has name
 				return new NamedObject(name, me, named);
			} else { //!named -> (internal: parent, but no name)
 				return new NamedObject(name, me, named);
			}
		} else {	//parent==null -> (root, array, hash, vect)
 			return new NamedObject(name, me, named);
		}

	}


	/**--------------------------------------------------------------------------
		* getArrayType: "ArrayOf-int" to "LI"
		* ------------
		* convert Array name string from:
		*		[*		//any number of '[' (including 0)
		*		(B|C|D|F|I|J|S|Z) |		//one of these represeting a primitive type
		*   (L[^;]*;)		//or the name of the class, in between 'L' and ';'
		**/
/*
	private static String getArrayType(Class clazz) {
		String a = "";	//will hold result
		String name = clazz.getName();
		if (DEBUG) System.err.println("Array type = \""+name+"\"");
		int i;
		for (i=0; name.charAt(i)=='['; i++)
			a += ARRAY_TOKEN;
		switch (name.charAt(i++)) {
			case 'B': a += "byte";
				break;
			case 'C': a += "char";
				break;
			case 'D': a += "double";
				break;
			case 'F': a += "float";
				break;
			case 'I': a += "int";
				break;
			case 'J': a += "long";
				break;
			case 'S': a += "short";
				break;
			case 'Z': a += "boolean";
				break;
			case 'L': a += name.substring(i, name.lastIndexOf(';'));
						//could also be name.length()-1
			break;
		}
		// NB: in a multi-dimensional array, Class.getComponentType() returns
		// the next one further in (which may also be an array).
		return a;
	}
*/




	/**--------------------------------------------------------------------------t
		* addAttr (map to primitives and String)
		* -------
		* Handles: byte, char, short, int, long, float, double and String.
		* As fields, arrays and custom data.
		* NOTE: String can never appear as custom data (can only be written as an
		* object).
		*
		* Consider: BIZARRE: Could do something odd for arrays: take the object
		* only as indicating the exact type of the array, and in here, when read
		* the length, actually create it.  And yet it makes sense in a way.
		*
		* or, return a proper "attrMsg", which includes the length as info.  This
		* is wha should be returned, anyway - not this hacked use of Attr.
		**/
	private ParserXML.Attr addAttr(Object o, boolean isArray) throws Exception {
		int length = -1;	//to flag if not set
		ParserXML.Attr attr;
		ParserXML.Attr name = null;
		while ( !(attr=p.readAttr()).isEnd ) {
			if (DEBUG) System.err.println(attr);
			if (attr.name.equals(XMLSerialize.NAME_TOKEN)) { //if name of this class
				if (name!=null)
					throw new ParserXML.ExceptionXML("just one name field");
				name = attr;
				continue;	//next attr
			}
				//ignore all fields of array, but for the name
			if (isArray) {	//length or primitive attributes
				if (attr.name.equals(XMLSerialize.LENGTH_TOKEN)) {	//if length
					if (INTERNAL_DEBUG) {
						System.err.println("ARRAY LENGTH: " +attr);
					}
					length = Integer.parseInt(attr.value);
					break;		//break off part way through (will this work?)
				}
				if (INTERNAL_DEBUG) {
					System.err.println("ARRAY PRIMITIVE ATTRIBUTES: " +attr);
				}
				//else, continues on, reading primitive values, and sticking them in.
				//assumed: .startsWith(XMLSerialize.ARRAY_PRIMITIVE_INDEX_TOKEN)
				String indexStr = attr.name.substring(XMLSerialize.ARRAY_PRIMITIVE_INDEX_TOKEN.length());
				int index = Integer.parseInt(indexStr);	//we know it is an int
				if (DEBUG) System.err.println(attr);
				Object value = getValue(o.getClass().getComponentType(), attr);
					//This also puts alias for String, because we can infer the type from
					//the field.  Not so with custom data.  I need to write this up.  The
					//immediate result is that we serialize internal Strings like objects
					//(in tags), like primitives (in attributes).
				Array.set(o, index, value);
			}
			else {	//if not array...
				if (attr.name.endsWith(XMLSerialize.ALIAS_STRING_TOKEN)) {//string alias
					if (INTERNAL_DEBUG) {
						System.err.println("STRING ALIAS: " +attr);
					}
					//(1). extract the fieldName
					String fieldName = attr.name.substring(0, attr.name.indexOf(XMLSerialize.ALIAS_STRING_TOKEN));
					//(2). get the value & use it to lookup the object
					Object strObj = aliasHash.get(attr.value);	//get the object (String)
					//(4). assign that object to the field
					Field f = getAllField(o.getClass(), fieldName);
					Class fc = f.getType();			//type specified by java, not by xml
					if (DEBUG) System.err.println("Made Field \""+fc.getName()+"\"");
					setFinal(f, o, strObj);
					//f.set(o, strObj);	//note: referring to already XML decoded object
				}
				else {	//if not string alias...
					if (attr.name.startsWith(XMLSerialize.INTERNAL_PRIMITIVE_TOKEN)) {
						if (INTERNAL_DEBUG) {
							System.err.println("INTERNAL: " +attr);
						}
						String primString = attr.name.substring(XMLSerialize.INTERNAL_PRIMITIVE_TOKEN.length());	//omit prefix (typically '_' or "_." or "someclass.")
						if (DEBUG) System.err.println(attr);
							//non-XML: we rely on the ordering of attributes
							//need (perhaps) to detect non-ordering, and at that time
							//start allowing for it....
							//or, we could detect the size, and add in nulls to match it.
							//And signal an error later.
							//NB: I think that Strings will always be done as tags, even
							//though attributes might be neater.
						primStore.add(attr.value);	//keep it in the universal form of "string"
						//IF we do readUTF() as a primitive, then we need to test the
						//type of it here, and put it into 
						//PROBLEM!!! We cannot tell the type of attributes!!!  Thus, we
						//cannot handle Strings differently!  We *could* add type
						//information, in the same hack that we use for alias strings; but
						//this is extra work, and is ugly.  We simply don't know until
						//the readObject() code is run, and the present buffer method
						//means that we cannot tell at all, unless we kept track of the
						//exact order of all custom data written... but this is lost 
						//in communication, because of the split between attr and tags...
						//Thus, we must change the format to do this, either utterly (so
						//that the order tells us), or slightly, to include type information
						//for Strings that are custom data.  Need to write a little white
						//paper on this!
							//Question: how are internal Strings stored?
							//It doesn't matter, except that they will be read with
							//readObject, and so should not be on the primStore...
							//There is also an alias issue...
					}
					else {	//if an ordinary field (not an array at all)
						if (INTERNAL_DEBUG) {
							System.err.println("ordinary: "+attr);
						}
						Field f = getAllField(o.getClass(), attr.name);	//subclass Field?
						Class fc = f.getType();			//type specified by java, not by xml
						if (DEBUG) System.err.println("Made Field \""+fc.getName()+"\"");
						setFinal(f, o, getValue(fc, attr));
						//f.set(o, getValue(fc, attr));
					}
				}
			}
		}
			//combine info from the name and final attr returned.  Awkward.
		attr.length = length;
		if (name==null) {
				//attr.value = null;	//some paths assume the name is here... ugly
			attr.nameMissing = true;	
			return attr;	//simple.  No name to return
		}
			//else, combine info from both (name and last attr)
		name.isEnd = attr.isEnd;
		name.emptyTag = attr.emptyTag;
		name.length = attr.length;
		return name;
	}


	/**--------------------------------------------------------------------------
		* getValue
		* ------------------
		* In: the class of the primitive; the attr (name-value pair)
		* Out: * object representing the primitive
		* Class may come from the "TYPE" field, or from array component type.
		* Used by:
		*		deserializeWrapper
		*		addAttr - note: an intermediate wrapper object is created redundantly
		*
		* It may be more efficient to not reuse this code.
		* when used for an attribute, it creates an Object needlessly;
		* however, such an object is required for Hashtable
		**/
	private Object getValue(Class fc, ParserXML.Attr attr) throws Exception {
		if (DEBUG) System.err.println("getting value of " +fc);
		if (fc.isPrimitive()) {     //PRIMITIVE
			if (fc==int.class) return new Integer(attr.value);	//most likely first
			else if (fc==double.class)
				return new Double(ParseUtilities.parseDouble(attr.value)); //second pr
			else if (fc==boolean.class) {
					return new Boolean(ParseUtilities.parseBoolean(attr.value));
			}
			else if (fc==byte.class) return new Byte(attr.value);
			else if (fc==char.class) {
				String a = decodeXML(attr.value);	//many chars may go to one
				if (a.length()!=1) throw new IllegalArgumentException("Character data must be exactly one character long; instead we got: '" +a+ "'");
				return new Character(a.charAt(0));
			}
			else if (fc==short.class) return new Short(attr.value);
			//else if (fc==int.class) return new Integer(attr.value));
			else if (fc==long.class) return new Long(attr.value);
			else if (fc==float.class)
				return new Float(ParseUtilities.parseFloat(attr.value));
			else throw new ParserXML.ExceptionXML(
				"Unimplemented primitive: \"" +fc.getName()+ "\"");
		}
		else {                      //CLASS
			if (fc==String.class) {
				String s = decodeXML(attr.value);
				putAlias(s);	//String is an object - not called for attr string
				return s;
			}
			else throw new ParserXML.ExceptionXML(
				"'"+attr.name+"' is not primitive or String");
		}
	}



	/**--------------------------------------------------------------------------
		* addTags
		* -------
		* Handles just tags (no attrs)
		* CONSIDER: is there an elegant way to reuse this code in Hashtable?
		* This is where the end tag is read
		* Perhaps pass in the open tag explicity, rather than (inefficiently)
		* recalling it from the object itself.  Why?  Because this doesn't work
		* for null objects.
		**/
	private void addTags(Object me) throws Exception {
		ParserXML.Tag tag;
		while ( ((tag=p.readTag())!=null) && (tag.start))	
			//createObject(tag.name, me);	//add this new tag to me
			createObject(tag, me);	//add this new tag to me //30Apr2001
//May10: NOTE!!!  We throw away the returned value, because it is already
				//plugged in for us! (done like this, because parent not known here)
				//Returned value is only used by: array, vector, hashtable
		if (tag==null)
			throw new ParserXML.ExceptionXML("Premature EOF");
	
		if (me==null) {		//dangling else!
			if (!tag.name.equals(XMLSerialize.NULL_TOKEN))	//special case of null
				throw new ParserXML.ExceptionXML(
					"</"+XMLSerialize.NULL_TOKEN+">","</"+tag.name+">");
		}
		else
			if (!tag.name.equals(me.getClass().getName()))
				throw new ParserXML.ExceptionXML(
					"</"+me.getClass().getName()+">","</"+tag.name+">");
	}


	/**--------------------------------------------------------------------------
		* addVectorElements
		* -----------------
		* Note: this should now be possible automatically, since we can read
		* private fields (and any internal serialization)
		* Note: Vector object already created when we get here.
		**/
	private void addVectorElements(Vector v) throws Exception {
		ParserXML.Tag tag;
		while (true) {
			Object obj;
			tag = p.readTag();
			if (tag==null) throw new Exception("Premature EOF");
			if (!tag.start) return;	//done!  caller will check if a correct endtag
			if (tag.name.equals("java.lang.String")	//bad: doubles up
					|| tag.name.equals("java.lang.Integer")
					|| tag.name.equals("java.lang.Double")
					|| tag.name.equals("java.lang.Boolean")
					|| tag.name.equals("java.lang.Character")
					|| tag.name.equals("java.lang.Byte")
					|| tag.name.equals("java.lang.Short")
					|| tag.name.equals("java.lang.Long")
					|| tag.name.equals("java.lang.Float")
				)
				obj = deserializeWrapper(tag, null).object;
			else
				//obj = createObject(tag.name, null);	//null=orphan object
				obj = createObject(tag, null);	//null=orphan object //30Apr2001
			if (DEBUG) System.err.println("Adding {"+obj+"} to Vector");
			v.add(obj);
		}
		//if (DEBUG) System.err.println("Vector: \n" +v);
	}

	/**--------------------------------------------------------------------------
		* addHashtableElements
		* --------------------
		* An container object.
		* Handles just tags (no attrs)
		**/
	private void addHashtableElements(Hashtable h) throws Exception {
		ParserXML.Tag tag;
		while (true) {
			Object[] objs = {null, null};
			for (int i=0; i<objs.length; i++) {		//{key, value}
				tag = p.readTag();
				if (tag==null) throw new Exception("Premature EOF");
				if (!tag.start)
					if (i==0) return; //end of hashtable
					else throw new Exception("incomplete Hashtable entry");
				if (tag.name.equals("java.lang.String")	//bad: doubles up
						|| tag.name.equals("java.lang.Integer")
						|| tag.name.equals("java.lang.Double")
						|| tag.name.equals("java.lang.Boolean")
						|| tag.name.equals("java.lang.Character")
						|| tag.name.equals("java.lang.Byte")
						|| tag.name.equals("java.lang.Short")
						|| tag.name.equals("java.lang.Long")
						|| tag.name.equals("java.lang.Float")
					)
					objs[i] = deserializeWrapper(tag, null).object;
				else
					//objs[i] = createObject(tag.name, null);	//null=orphan object
					objs[i] = createObject(tag, null);	//null=orphan object//30Apr2001
			}
			if (DEBUG) System.err.println("Putting {"+objs[0]+", "+objs[1]+"}");
			h.put(objs[0], objs[1]);
		}
		//if (DEBUG) System.err.println("Hashtable: \n"+h);
	}

	/**--------------------------------------------------------------------------
		* deserializeClass
		* ----------------
		* Handles the attrs.
		* Need to refactor the test for wrapper type
		* Reads the whole thing in, including <x></x> or <x/>
		**/
	private NamedObject deserializeClass(ParserXML.Tag startTag, Object cheatParent) throws Exception {
		ParserXML.Attr attr, end, name = null;
		ParserXML.Tag tag;
		attr = p.readAttr();	//p is the ParserXML.
		if (DEBUG) System.err.println("deserializeClass: "+ attr);
		if (attr.name.equals(XMLSerialize.NAME_TOKEN)) {
			//System.err.println("Got a name in a wrapper: "+startTag.name+" "+attr.value+";");
			name = attr;
			attr = p.readAttr();
		}
		if (!attr.name.equals(XMLSerialize.CLASSNAME_TOKEN)) 
			throw new ParserXML.ExceptionXML(XMLSerialize.CLASSNAME_TOKEN+ " expected");
		if (!(end = p.readAttr()).isEnd)
			throw new ParserXML.ExceptionXML("\">\" or \"/>\" expected");
		if (!end.emptyTag) {
			tag = p.readTag();
			if (!tag.name.equals(startTag.name) || tag.start)
				throw new ParserXML.ExceptionXML("</"+tag.name+"> expected");
		}
		//IDEA: could make up the expected tag, and use Tag.equals() to compare
		if (DEBUG) System.err.println("String read in: " + attr.value);

		Class clazz =  Class.forName(attr.value);
		putAlias(clazz);	//String is an object
		if (name!=null)	{ //can only happen for Strings... (ugly hack!)
			Field f = getAllField(cheatParent.getClass(), name.value);
			setFinal(f, cheatParent, clazz);
			//f.set(cheatParent, clazz);	//Custom "Class" object code...
 			return new NamedObject(name.value, clazz, true); //name!=null -> true
		}
 		return new NamedObject(null, clazz, false); //name!=null -> false
	}


	/**--------------------------------------------------------------------------
		* deserializeWrapper
		* ------------------
		* Handles the attrs.
		* Need to refactor the test for wrapper type
		* Reads the whole thing in, including <x></x> or <x/>
		**/
	private NamedObject deserializeWrapper(ParserXML.Tag startTag, Object cheatParent) throws Exception {
		ParserXML.Attr attr, end, name = null;
		ParserXML.Tag tag;
		attr = p.readAttr();	//p is the ParserXML.
		if (DEBUG) System.err.println("deserializeWrapper: "+ attr);
		if (attr.name.equals(XMLSerialize.NAME_TOKEN)) {
			//System.err.println("Got a name in a wrapper: "+startTag.name+" "+attr.value+";");
			name = attr;
			attr = p.readAttr();
		}
		if (!attr.name.equals(XMLSerialize.VALUE_TOKEN)) 
			throw new ParserXML.ExceptionXML(XMLSerialize.VALUE_TOKEN+ " expected");
		if (!(end = p.readAttr()).isEnd)
			throw new ParserXML.ExceptionXML("\">\" or \"/>\" expected");
		if (!end.emptyTag) {
			tag = p.readTag();
			if (!tag.name.equals(startTag.name) || tag.start)
				throw new ParserXML.ExceptionXML("</"+tag.name+"> expected");
		}
		//IDEA: could make up the expected tag, and use Tag.equals() to compare
		if (DEBUG) System.err.println("String read in: " + attr.value);
		if (startTag.name.equals("java.lang.String")) {	
			String s = decodeXML((String)attr.value);	//not attr.name! Don't forget to decode!
			putAlias(s);	//String is an object
			if (name!=null)	{ //can only happen for Strings... (ugly hack!)
				Field f = getAllField(cheatParent.getClass(), name.value);
				setFinal(f, cheatParent, s);
				//f.set(cheatParent, s);
 				return new NamedObject(name.value, s, true); //name!=null -> true
			}
 			return new NamedObject(null, s, false); //name!=null -> false
		}
		Class c = Class.forName(startTag.name);
		try {
			Field f = getAllField(c, "TYPE");
			Object o = getValue((Class)f.get(c), attr);
			putAlias(o);	//NOTE: *not* called if a String, so not inc twice
 			return new NamedObject(null, o, false); //can never be named
		} catch (NoSuchFieldException e) {
			System.err.println("'" +c+ "' is not a wrapper class: "+e);
		}
		throw new Exception("Shouldn't get here");
	}


	/**--------------------------------------------------------------------------
		* decodeXML
		* ---------
		* &amp; etc.
		* To do: this also needs to be applied to char (not just Strings)
		* should rewrite to use StringBuffer - it presently creates new objects
		* like mad - a very expensive operation.
		**/
	private String decodeXML(String in) {
		String a = new String("");	//prevent reference sharing for zero length
		int i = 0;
		try {
			for (i=0; i<in.length(); i++) {
				char c = in.charAt(i);
				if (c=='&') {				//if the escape character
					i++;
					if (in.startsWith("amp;", i))       {a+='&';  i+=3;} //NB: loop's i++
					else if (in.startsWith("lt;", i))   {a+='<';  i+=2;}
					else if (in.startsWith("gt;", i))   {a+='>';  i+=2;}
					else if (in.startsWith("apos;", i)) {a+='\''; i+=4;}
					else if (in.startsWith("quot;", i)) {a+='"';  i+=4;}
				}
				else if (c=='\\') {	//if \ escape char. NB: \ is quoted as \\ in source
					i++;
					if (in.charAt(i)=='u') { //NB: loops's i++ if \ u (\ then u)
						i++;
						a+=(char)Integer.parseInt(in.substring(i,i+4),16);
 						i+=3;
					}
					else if (in.charAt(i)=='\\') { a+='\\'; i+=0; }	//if \\ (\ then \)
				}
				else {
					a += c;
				}
			}
		} catch (StringIndexOutOfBoundsException e) {
			System.err.println( "Seems to be an incomplete escaped entity or \\unnnn: "
				+in.substring(i) );
		} catch (IndexOutOfBoundsException e) {
			System.err.println( "Seems to be an incomplete escaped entity: "
				+in.substring(in.lastIndexOf('&')) );
		} catch (NumberFormatException e) {
			System.err.println( "Seems to be a faulty \\unnnn escaped control char: "
				+in.substring(i) );
		}
		
		return a;
	}


	/**--------------------------------------------------------------------------
		* Test class
		* ----------
		* Predates ObjIn and ObjOut!
		**/
	public static class Test {
		public static void main(String args[]) throws Exception {

			XMLDeserialize d = new XMLDeserialize();	//defaults to System.in
			XMLSerialize s = new XMLSerialize();	//defaults to System.out
			Object o;
			while ( (o = d.deserialize()) != null) {	//can loop for streamed XML
			/*
				System.err.println("Printing created object:");
				System.err.println();
				System.err.println(o);
				System.err.println();
				System.err.println("(Re)serializing created object:");
				System.err.println();
			*/
				s.serialize(o);
			}
		}
	}
}
