/** (c) Brendan Macmillan 2001, licensed under GNU's GPL 2 */
/** (Also available for non-GPL, commercial use, for fee) */
/** Website: http://www.csse.monash.edu.au/~bren/JSX
/** Website: http://freshmeat.net/projects/jsx


/** ===========================================================================
  * XMLSerialize (JSX0.8.8.4).
	* ============
	* 
	* NB: This file best viewed with tabs set to 2.
	* For vim (vi clone):
:set ts=2
	* For emacs:
M-x edit-tab-stops
	* and then put a ":" in every second position (use space to overwrite).
	* (M is "Meta key"; usually just Alt)
	*
	* DESCRIPTION
	* Pass in an object, and it writes out a serialized version, in XML format.
	* It mimics Java's own serialization, and so can handle almost all classes,
	* including those with:
	*		- Private fields
	*		- A lack of a no-argument constructor
	*		- Their own internal serialization (except for Externalizable)
	*	It handles:
	*		- Circular references
	*		- Multi-dimensional arrays (ie arrays of arrays)
	*
	* For comments of any kind
	* (eg bug reports, bug fixes, suggestions, patches, criticism, comments,
	* insights, ideas, frustrations, irritations, or alternative software in the
	* same space):
				bren@csse.monash.edu.au  <G. Brendan Macmillan>
	**/

/** BUGS.
	*		Wrapper classes only serialized correctly when within container types
	*			- Bug fixed or not?
	*			- would it now work automatically?
	*		String as first object?
	**/

/** The mapping of Java to XML.
	*		Primitives map to attributes.
	*		References map to elements:
	*		- The element name is the *runtime class*.  eg <java.util.Hashtable>
	*		- A NAME_TOKEN (eg "obj-name") attribute is the name of the field, if it
	*		has a name.
	*		- NB: NAME_TOKEN is illegal as a Java field (because of '-'), so no
	*		namespace conflict.
	**/

/** Logic flow.
	*		For each class: (serializeObject)
	*			write all fields as attributes - if primitive (serializeFields)
	*			write all fields as elements - if reference (serializeReferences)
	*				-> recurse
	**/

/**	Notes.
	* NB: Newlines within strings are outputted *exactly* as they are stored
	*	in the string.  This might not be what you want, when moving across DOS
	* (/n/r) and UNIX (/n).
	**/

/** To do.
	*		- nice, informative error messages when the field is not found.
	**/

/** Circular References.
	*	Increment a count whenever an object is serialized - the aliasSerialNumber.
	*
	* This number becomes the alias (or alternate name) for that object.
	*	It is the *very* simplest implementation; not very human readable, but
	* will reveal implementation difficulties without complicating them.
	* NB:
	*		- A null value is not an object (so doesn't increment the count)
	*		- "String ambivalence": String are objects, but encoded as attributes
	*		Could just say <fieldname>-alias="<aliasname>"
	*		Similarly, could say <fieldname>-null="" for null (!= to empty string!)
	*		Difficulty is extra parsing time to detect the "-".  OK if can do it
	*			efficiently, which would be good, anyway.
	*		- A normal object is a an object.
	*		- A wrapper object is an object.
	*
	* In present implementation, aliases are needed in
	*		- serializeObject,
	*		- serializeWrapper,
	*		- serializeFields - for strings (strings in containers are in wrappers)
	*	NB: it is pre-inc.  Begins at -1, and is PRE-INC just before each use.
	**/

/** Handling internal serialization (ObjOut.java ObjIn.java).
	*	(1). When a class specifies its own serialization, it only does it for
	*	itself, not its ancestors (superclasses).  Therefore, the test for a
	*	writeObject() method must be in getAllFields(), which walks up the
	*	inheritance tree.
	*	(2). This is presently called twice for each object: once for primitives,
	*	and once for objects (references).  This is because JSX encodes these
	* very differently in XML (attributes vs. tags), though this is not the
	*	only implementation.  We now pass in a flag so that the internal
	*	serialization logic is only executed once (in the first call, which comes
	* when serializing primitives (as attributes).
	* (3). A class's internal serialization invokes the whole process again
	* for objects (references), using writeObject() (in ObjOut.java),
	* with no parameters to indicate that it is not a fresh level serialization.
	* This is important to know, for reseting the alias table, for correct
	* tabbing in formatting, and for whether to save up the Objects until
	* after the primitives are done, instead of doing them right away.  These
	* two cases of fresh vs. internal are distinguised by:
	*		(a). serialize invocation (not serializeObject, which nested tags invoke)
	*		(b). invocationCount != 0
	*	The primitive writes, such as writeInt() are easily distinguished, as that
	* is the only way they may legitimately be invoked (for JSX, anyway).
	*	(4). Thus, the internally serialized objects and primitives are gathered
	*	and collected in a bucket.
	*	(5). For primitives, it would be nice for this bucket to subclass Field,
	* so they could remain invisible to the primitive-handling method
	*	serializeFields().  However, Fields are "final", so instead we pass back
	*	an array of type Object, and serializeFields() must distinguish between
	*	them with the instanceof operator, a name, value String pair (String[]?)
	*	Or class x (String name; String value;}?  putField() is no good.
	*	(6). NB: No recursion occurs from this internal serialization - primitives
	* don't recurse, and the object references are merely noted, not written
	* at this point.  Thus, the invoked writeObject() is guaranteed to return
	*	immediately.
	* (7). Thus, we can use the stored primitives and objects immediately (like
	*	a one-level stack).   The primitives are written as above; and the 
	*	object references are serialized just before the call to
	* serializeReferences.
	* (8). NB: This call to serializeReferences() will invoke getAllFields() a
	*	second time, and we don't wish to invoke the internal serialization again,
	* or it will be awkward to distinguish the first and second lot of stored
	*	objects.  However, [NB] when we implement defaultWriteObject(), we will
	*	still need to detect (but not invoke), so we don't serialize twice.
	* (9). We will need to start fresh for each invocation of an internal
	* serialization, so mix up the storage of primitives and object references
	* from different classes.  Note that this can be true even within the one
	* object, [NB] when we implement internal serialization of ancestor
	*	superclasses (which will be needed we when use Hashtable's internal
	* serialization instead of the present ad hoc approach, and we want to allow
	* users to subclass it - as per Michael Oswall's request.
	**/

package JSX;
import java.lang.reflect.*;
import java.io.*;
import java.util.*;	//for Hashtable

public class XMLSerialize {
	static final boolean DEBUG = false;	
	//static final boolean DEBUG = true;
	static final boolean DEBUG_HASH = false;	//Hashtable uses .equals(), not ==
	//static final boolean DEBUG_HASH = true;
	static final boolean TESTCIRC = false;
	//static final boolean TESTCIRC = true;	//circular references
	static final boolean INTERNAL_DEBUG = false;
	//static final boolean INTERNAL_DEBUG = true;	//internal serialization
	static final boolean CACHE_DEBUG = false;
	//static final boolean CACHE_DEBUG = true;//messy multi invoke of getAllFields
	static final boolean REENTRY_DEBUG = false;
	//static final boolean REENTRY_DEBUG = true; //when serializeObject() called


	static final int ALIAS_INIT = -1;//so pre-increment in serializeObject
	private int aliasSerialNumber;	//simplest circular reference names

	static final String ALIAS_TAG_TOKEN = "alias-ref";
	static final String ALIAS_ATTR_TOKEN = "alias";	//'-' not needed, b/c alone
	static final String ALIAS_STRING_TOKEN = "-alias";

	static final String VERSION = "Version: JSX0.8.13 (maybe out of date - sorry! See the name of the jar.)";	//MUST remember to update this!

	/**	
		*	"-" avoids namespace conflicts between serialization flags and 
		*	actual class or field names.
		*	"-" legal XML attr; illegal java field; illegal java class
		**/
	//static final String NAME = "is-the";	//"-" legal attr; illegal field
	//static final String NAME = "class-of";	//opposite of "instanceof"
	//static final String NAME = "run-type-of";	//opposite of "instanceof"
		//there is an issue with compatibility here!  I keep changing it, and old
		//serializations are no longer compatible with it.
	static final String NAME_TOKEN = "obj-name";

	static final String CLASSNAME_TOKEN = "className";
	//static final String ARRAY_TOKEN = "A-";
	//static final String ARRAY_TOKEN = "[]";	//not allowed as XML names!
	static final String ARRAY_TOKEN = "ArrayOf-";
	//static final String ARRAY_PRIMITIVE_INDEX_TOKEN = "a-";
		//NB: prefixes needed - attr names can't start with a numeral
	static final String ARRAY_PRIMITIVE_INDEX_TOKEN = "a";	//'-' not needed
	static final String INTERNAL_PRIMITIVE_TOKEN = "_.";	//'-' not needed
		//NB: '_' is perfect to convey the sense of an internal name!
		//This may also be appropriate for arrays; but 'a' for array, and no
		//	other attributes to confuse (except name and length)
		//Problem: a valid java fieldname.  Although _0 _1 _2 (ie with numerals)
		//	are unlikely to be used as fieldnames, the simplist parsing is
		//	confused by fieldnames such as "_name".  Using the prefix "_."
		//	should fix this.  This bug was discovered by Thomas Hiller, who
		//	thus became the third successful JSX Challenger, on:
		//	Tue, 16 Jan 2001 17:58:24 +0100
		

	static final String LENGTH_TOKEN = "length";
	static final String VALUE_TOKEN = "valueOf";
	static final String NULL_TOKEN = "null";

	static final String SPACER = "  ";		//\t

	static final int NOT_SERIALIZED =
		Modifier.STATIC | Modifier.TRANSIENT;
//		| Modifier.FINAL; //todo 26June2001: be able to set final fields


	public static void main(String ars[]) {
		System.err.println(VERSION);
	}
	/**--------------------------------------------------------------------------
		* API
		* ===
		* CONSTRUCTORS
		* ============
		**/
	private PrintWriter out;
	public XMLSerialize() throws IOException {	//defaults to System.out
		this(new PrintWriter(new OutputStreamWriter(System.out), true));
	}
	public XMLSerialize(PrintWriter out) throws IOException {
		this.out = out;
	}


	//legacy method
	public void serialize(Object o, PrintWriter newOut) throws IOException {
		out = newOut;	//can change output destination between invocations
		commonSer(o);
	}
	public void serialize(Object o) throws IOException{
		commonSer(o);
	}
	/** ------------------------------------------------------------------------
		* commonSer()
		* -----
		* Heart.
		* Consider: should this be static?
		**/
		 /* Mechanism for internal serialization to pass out objects.
			* Consider: reimplementing with faster ArrayList (not synch).
			* Danger: incremental capacity increases.
			* Consider: doubling concept used in OOS.
			*/
	int invocationCount = 0;
	private void commonSer(Object o) throws IOException{
		if (invocationCount!=0) {		//Exception if no objStore exists.
			objStore.add(o);		//for internal serialization.  NB: doesn't recurse.
			return;
		}
		else {
			reset();
			invocationCount++;
			serializeObject(o, null, "");
			out.println();
			if (TESTCIRC) {
				System.err.println("Count was: " + (aliasSerialNumber-ALIAS_INIT));
				System.err.println();
			}
			invocationCount--;
		}
	}



	/** ------------------------------------------------------------------------
		* METHODS
		* =======
		**/
	/** ------------------------------------------------------------------------
		* reset()
		* -----
		**/
	public void reset() {	//can reset - important part of API!
		if (invocationCount!=0)
			throw new Error("Serialiize: Attempt to reset alias table mid-way.");
		defaultFlag = false;	//should do this at the start of each object...
		primStore = new Vector();
		initAlias();
	}
	/** ------------------------------------------------------------------------
		* defaultWriteObject()
		* ------------------
		* Forwarded from ObjOut.
		* NB: no object specified.  It is merely returning a flag.
		**/
	private boolean defaultFlag; //return value. "default" is keyword
	private Vector primStore;	// = new Vector();
	Vector objStore;	//NB: a new one created for each
	public void defaultWriteObject() {
		if (DEBUG)
			System.err.println("defaultWriteObject() in XMLSerialize: should store it");
		defaultFlag = true;
	}	
	public void close() { out.close(); }
	public void flush() { out.flush(); }

	/** ------------------------------------------------------------------------
		* writePrim(str)
		* ------------------
		* Forwarded from ObjOut.
		* Caller (ObjOut) must convert each of the primitive types to String
		* first.  This is trivial eg: (x+"")
		**/
	public void writePrim(String str) {
		if (DEBUG)
			System.err.println("writeInt(i) in XMLSerialize: should store it");
		primStore.add(str);
	}


	/**--------------------------------------------------------------------------
		* alias nascent-object
		* --------------------
		* collect init into one routine... suggests it should be a class.
		* change to public, so ObjOut can access it
		**/
	private Hashtable aliasHash = new Hashtable();	//reuse the one
	private void initAlias() {
		aliasSerialNumber = ALIAS_INIT;
		aliasHash.clear();
	}
	/**--------------------------------------------------------------------------
		* put Alias
		* --------
		*	inline = !serialized;
		* if (inline)
		*		//add to hash
		*	else
		*		//write as alias
		*	return (inline);		//to tell caller to serialize it inline!
		*		//else, caller leaves the ref writing up to this routine
		**/
	//simplest way to circumnavigate the ".equals" instead of identity test 
	//of Hashtable (in Java 1.2)
	class IdObject {
		Object o;
		public int hashCode() {
			return System.identityHashCode(o);
		}
		public boolean equals(Object x) {	//essense
			return ((IdObject)x).o == this.o;	//assumption that used by Hashtable,
				//and that the tested part (key) will always be an IdObject
		}
		public IdObject(Object o) {
			this.o = o;
		}
	}
	boolean areAliasesUsed=true; //default "on"; set in constructor
		//this name feels confusing to me...
	//Return value: false = I didn't add it (already there); don't inline it.
	//true = created new alias' you need to serialize it inline.
	private boolean putAlias(Object o, String fieldName, boolean isStringAttr, String indent) {
		//if (!areAliasesUsed)
		//	return true;
		if (DEBUG_HASH) System.err.println("  entering putAlias with "+o);

		if (o==null) return true;	//carry on
		Object alias;	//will be String, but save the conversion
			if (DEBUG_HASH) {
				for (Enumeration keys=aliasHash.keys(); keys.hasMoreElements();) {
					Object a = keys.nextElement();
					System.err.println(a + " -- " + a.getClass() + " -- " +a.hashCode());
				}
				System.err.println("looking for: "+o+" -- "+o.getClass() + " -- " +System.identityHashCode(o));
				System.err.println("Got: "+myget(o));
				System.err.println("Same object? 1 " + (myget(o)!=null) );
				System.err.println("Same object? 2 " + (mycontainsKey(o)) );
			}
		if ( (alias = myget(o))==null) {
			++aliasSerialNumber;	//pre-inc (use this syntax to highlight it)
			myput(o, aliasSerialNumber+"");	//add (convert to String)
				if (DEBUG_HASH) System.err.println("  recording new alias: "+indent +"inline " +aliasSerialNumber+ ":");
			//Need to:
			//	indicate "inline" somehow.
			//	write as inline (same as now).
			return true;	//yes, should serialize the object inline
		}
		else
		{
			if (isStringAttr) {
				printAttr(fieldName + ALIAS_STRING_TOKEN, alias+"");
			}
			else {
					if (DEBUG_HASH) System.err.println("  repeating old alias: "+indent+ "alias " +alias+ ":");
				printOpenTag(ALIAS_TAG_TOKEN, indent);
				if (fieldName!=null) printAttr(NAME_TOKEN, fieldName);	//nice to reuse
				printAttr(ALIAS_ATTR_TOKEN, alias+"");
				out.println("/>");		//empty tag - under control of caller
			}
			return false;	//no, don't serialize inline; we (or you?) write the alias
		}
	}
	private void removeAlias(Object key) {
		if (areAliasesUsed) {	//if aliases are used, we don't mess with them
		} else {	//else, we remove them, so only the path remains.
			try {
				Object oo = aliasHash.remove(new IdObject(key));
				if (oo==null)
					throw new Exception("Try to remove an object not in aliasHash: " +key);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	public Object myput(Object key, Object value) {
		return aliasHash.put(new IdObject(key), value);	//just a dumb wrapper
	}
	public Object myget(Object key) {
		return aliasHash.get(new IdObject(key));	//just a dumb wrapper
	}
	public boolean mycontainsKey(Object key) {
		return aliasHash.containsKey(new IdObject(key));	//just a dumb wrapper
	}



	/**--------------------------------------------------------------------------
		* Object (map to an Element)
		* ------
		* tag (name of element) is always the runtime class (type) of the object.
		* SUGGEST: write open and close tags separately; but problem of attributes
		*  check: Wrapper classes here (String, primitive wrappers)
		*  check: container classes (like Hashtable)
		*
		* NB: aliasSerialNumber++ three times, before of nature of sub-arrays.
		**/

	Class[] OOS_ARGS = {ObjectOutputStream.class};	//works fine!
		//just mispelling?  it was: "ObjectOutpuStream.class"
	public Object[] writeObjectArglist;  //set by caller
	/**--------------------------------------------------------------------------
		* setArg()
		* -------
		* Consider: putting this in constructor; or caller in same package or same
		* class.
		* THIS IS TRICKY!!! MUST BE CHANGED!!!
		**/
	public void setArg(Object[] argList) {
		writeObjectArglist = argList;
	}

//	private void writeObject(ObjectOutputStream oos) {
//	}
	private void serializeObject(Object o, String fieldName, String indent) 	throws IOException {
		if (REENTRY_DEBUG) {
			System.err.println("-----------------------");
			if (o!=null) {
				System.err.println(o);
				System.err.println(o.getClass());
			}
		}
		//NOTE: null does not get an alias
		if (o==null)	{ //an array object can also be null...
						//in which case we lose the type; but null is only of type "null"
			printOpenTag(NULL_TOKEN, indent);
			if (fieldName!=null) printAttr(NAME_TOKEN, fieldName);	//nice to reuse
			out.println("/>");		//good to have control of the newline here
			return;	//a null object can have no children
		}
	/*
		try {
			if (!(o instanceof Serializable))	//mark such fields "transient"
				throw new NotSerializableException(o.getClass().getName());
		} catch (Exception e) { System.err.println(e); return; }
	*/
		String runTimeClass = o.getClass().getName();
		Class clazz = o.getClass();

//FACTOR this out into a separate method, so we can recurse nicely.
		if (clazz.isArray()) {
			serializeArrayReferences(o, fieldName, indent);
			//array is dealt with internally
		}

		else if (clazz==Class.class) {
			serializeClass((Class) o, fieldName, indent);
			//each time a string is encoded, it becomes a new object?
		}

		else if (clazz==String.class) {	//TODO: deserializer needs to use this too.
			serializeWrapper(o, fieldName, indent);
			//each time a string is encoded, it becomes a new object?
		}

		else if (clazz==Vector.class) {
			if (putAlias(o, fieldName, false, indent)) {
				printOpenTag(runTimeClass, indent);
				if (fieldName!=null) printAttr(NAME_TOKEN, fieldName);	//nice to reuse
				if (((Vector)o).size() > 0) {
					out.println(">");		//move into ParserXML?
					serializeVectorReferences((Vector)o, indent);	//could poly
					printCloseTag(runTimeClass, indent);
				} else {
					out.println("/>");		//empty tag
				}
				removeAlias(o);	//only acts when in "min aliases" mode
			}//putAlias
		}
		else if (clazz==Hashtable.class) {
			if (putAlias(o, fieldName, false, indent)) {
				printOpenTag(runTimeClass, indent);
				if (fieldName!=null) printAttr(NAME_TOKEN, fieldName);
				if (((Hashtable)o).size() > 0) {	//same as Vector
					out.println(">");		//move into ParserXML?
					serializeHashtableReferences((Hashtable)o, indent);	//poly out; nicer
					printCloseTag(runTimeClass, indent);
				} else {
					out.println("/>");		//empty tag
				}
				removeAlias(o);	//only acts when in "min aliases" mode
			}//putAlias
		}

		//a Class Instance
			//What if root object *is* a wrapper class?
		else {
			//8Aug2001 - we need to do writeReplace before checking the alias.
			//The replacement object is decided dynamically - we can't predict it
			//from the instance which is replaced.  From the point of view of the
			//XML stream, it is as if it the replacing object was the only one
			//that ever existed.
			//8Aug2001 - therefore, we call writeReplace now.  It's simple:
			o = writeReplace(o); //in case change, update info:
			runTimeClass = o.getClass().getName();
			clazz = o.getClass();
			if (putAlias(o, fieldName, false, indent)) {
				printOpenTag(runTimeClass, indent);
				if (fieldName!=null) printAttr(NAME_TOKEN, fieldName);
				serializeFields(o, indent);	//NB: also gathers internal writes
				if (hasChildren(o)) {
					out.println(">");		//move into ParserXML?
	 			 /*  NB: we do internal first, so parser knowes when finished
					* internal serialization
					* ASSUME:
					* (0). We ignore Hashtable and Vector (use old ad hoc methods)
					* (1). defaultWriteObject() does nothing (or we get double header)
					*/
					if (INTERNAL_DEBUG) {
						System.err.println("internal objects:");
					}
					Vector localObjStore = objStore;	//NB: local -> stack; for recursion
					Vector localDefaultCache = defaultCache;	//NB: local -> stack; for recursion
					for (Enumeration keys=localObjStore.elements(); keys.hasMoreElements();) {
						Object a = keys.nextElement();
						if (INTERNAL_DEBUG) {
							if (a!=null) {
								System.err.println("in objStore: "+a);
								System.err.println(a.getClass());
							}
						}
						serializeObject(a, null, indent+SPACER+SPACER);	//debug print out
					}
						//objStore.clear(); //NB: reset when consumed
						//try: No need - now a new one each time
					if (INTERNAL_DEBUG) {
						System.err.println("should be cleared: "+localObjStore.size());
							//do internal first, so parser knows end automatically.
						System.err.println("references:");
					}
					defaultCache = localDefaultCache;	//restore! (count is OK)
					serializeReferences(o, indent);	//NEED to use default... how?
					printCloseTag(runTimeClass, indent);
				} else {
					out.println("/>");		//empty tag
				}
				//8Aug2001 - the object has now been completely written.  It is too
				//late to call writeReplace
				removeAlias(o);	//only acts when in "min aliases" mode
			}//putAlias
		}

	}

	/**--------------------------------------------------------------------------
		* I wonder if these tag factorings should be in ParserXML?  It's not
		* parsing - quite the opposite - but it is at the same level as the parser.
		* In the parlace of Compilers, it is "code generation", I guess
		**/
	private void printOpenTag(String tag, String indent) {
		out.print(indent + "<" +tag);
	}
	private void printCloseTag(String tag, String indent) {
		out.println(indent + "</" +tag+ ">");
		//out.println();	//nicer formatting	hmmm... also used by Wrapper...
	}

/** ---------------------------------------------------------------------------
	* writeReplace
	* ------------
	* Reflectively invoke the present objects's writeReplace().
	* From the perspective of the XML stream, the replacement object (the one
	* that this routine returns) is the only one that ever existed.
	* "parameterless" = no-arg
	* This routine has a pattern that seems *very* similar to readResolve().
	* It may be worth factoring something out.  Defer until another 
	* reflexive invocation is required.
	**/
	Object writeReplace(Object me) throws ObjectStreamException {
		try {
			Class replacedClass = me.getClass();
			Method writeReplaceMethod = replacedClass.getDeclaredMethod("writeReplace", new Class[]{});
			writeReplaceMethod.setAccessible(true);
			Object newme = writeReplaceMethod.invoke(me, new Object[]{}); //no arg
			if ( !(me.getClass().isAssignableFrom(newme.getClass())) )	//ie go back
				throw new ClassCastException("The writeReplace() method of class '"+
					replacedClass.getName()	+"' returned an object of type '"+
					newme.getClass().getName() +"'.");
					//would it be more efficient to initialize all these exceptions at
					//load time?  Or better to create JIT, only as needed?
			return newme;
		} catch (NoSuchMethodException e) {
			//Not an error - just means that it doesn't implement readResolve
			return me;	//if no writeReplace - the object is simply unchanged!
		} catch (IllegalAccessException e) {
			System.err.println("IllegalAccessException - this shouldn't happen, since we did setAccessible().  Perhaps security on the JVM precludes it?");
			e.printStackTrace();
			return me;	//just continue on - what would be a better thing to do?
		} catch (InvocationTargetException e) {
			System.err.println("JSX InvocationTargetException:");
			//e.printStackTrace();	//not sure how this should be handled...
			//is this the best way to handle it?  What does Serialization do?
			//Probably best to throw whatever is thrown.
			try {
				throw (ObjectStreamException) e.getTargetException(); //uncork it
				//contract of writeReplace guarantees that it only throws this class
				//However, neither the compiler nor ObjectOutputClass checks this.
				//Since this is a compile-time kind of thing, it is reasonable that
				//ObjectOutputClass does not check it, but only runtime problems...
				//Is the situation different for things like readObject() being private?
			} catch (ClassCastException f) {
				throw new Error("writeReplace() of "+me.getClass()+ " threw an '"+e.getTargetException()+"'. It should only be of type ObjectStreamException");
			}
			//unreachable - can't to get here.
		}
		//let all other Exceptions through - esp ClassCastException! - for caller
		//to handle.
	}


	/**--------------------------------------------------------------------------
		*	serializeArrayReferences
		*	------------------------
		* Can contain primitives (no wrapper) or objects
		* If called on non-array, it serializes as an Object (needed for recursion)
		**/
	private void serializeArrayReferences(Object o, String fieldName, String indent) throws IOException{
		Class clazz;
		if (o==null || !( (clazz=o.getClass()).isArray() )) {
	//		System.err.println("Just about do output the base type element of an array");
			serializeWrapperOrObject(o, indent);
			return;		//bottoms out.
		}
		if (!putAlias(o, fieldName, false, indent))
			return;	//don't inline if already done
		String runTimeClass = o.getClass().getName();	//redundancy here (see above)
		//out.println(indent + "ARRAYS NOT YET IMPLEMENTED");
		runTimeClass = getArrayType(clazz);
		printOpenTag(runTimeClass, indent); //map "[" to "ArrayOf-" etc
		if (fieldName!=null) printAttr(NAME_TOKEN, fieldName);
		int length  = Array.getLength(o);
		printAttr(LENGTH_TOKEN, length+"");
/*
	//we comment out this section, because zero-length arrays in the emptyTag
	//format are are not being deserialized correctly.  Disabling the emptyTag
	//option is the simplest hack to fix it, as some users need it urgently.
	//Many thanks to 
	//	Raimund 'Raimi' Jacob  - Mon, 29 Jan 2001 00:47:58 +0100 (MET)
	//	Gary Lawrence Murphy  - Thu Feb  1 12:37:44 2001
	// for identifying this bug and help testing ways to solve it.
		if (length==0)		//if o.length==0
			out.println("/>");		//empty tag
		else
*/
		{		//<- this brace remains from the commented-out if-else above.
				//if primitive, put it here?
			//if *last* array (ie array of non-array), and also of a primitive.
			//well, to get here, must be an array...
			//...so check check to see if it as array of a primitive
			if ( clazz.getComponentType().isPrimitive() ) {
				for (int i=0; i<length; i++) {
					if (clazz.getComponentType() == byte.class) { //if byte[]
						 //binary data special case. //Thanks to Mark Collette.
					} else {
						out.println();
						out.print(indent);
					}
					if (clazz.getComponentType()==char.class)	//primitive, not wrapper
						printAttr(ARRAY_PRIMITIVE_INDEX_TOKEN+i, encodeXML(Array.get(o,i)+""));
					else
						printAttr(ARRAY_PRIMITIVE_INDEX_TOKEN+i, Array.get(o,i)+"");
				}
				out.println("/>");		//*cannot* contain other tags. (into ParserXML?)
			}
			else {	//ie component is a reference type (another array or an object)
				out.println(">");		//move into ParserXML?
				for (int i=0; i<length; i++) {		//actually a recursion
					serializeArrayReferences(Array.get(o, i), null, indent+SPACER);
				}
					//closeTag is inside
				printCloseTag(runTimeClass, indent);
			}
		}
		removeAlias(o);
	}

	/** convert Array name string from:
		*		[*		//any number of '[' (including 0)
		*		(B|C|D|F|I|J|S|Z) |		//one of these represeting a primitive type
		*   (L[^;]*;)		//or the name of the class, in between 'L' and ';'
		**/
	private String getArrayType(Class clazz) {
		String a = "";	//will hold result
		String name = clazz.getName();
		if (DEBUG) System.err.println("Array type = \""+name+"\"");
		int i;
		for (i=0; name.charAt(i)=='['; i++)
			a += ARRAY_TOKEN;
		switch (name.charAt(i++)) {
			case 'B': a += "byte"; break;
			case 'C': a += "char"; break;
			case 'D': a += "double"; break;
			case 'F': a += "float"; break;
			case 'I': a += "int"; break;
			case 'J': a += "long"; break;
			case 'S': a += "short"; break;
			case 'Z': a += "boolean"; break;
			case 'L': a += name.substring(i, name.lastIndexOf(';')); break;
						//could also be name.length()-1
		}
		// NB: in a multi-dimensional array, Class.getComponentType() returns
		// the next one further in (which may also be an array).
		return a;
	}

	/**--------------------------------------------------------------------------
		*	hasChildren
		*	-----------
		*	An inefficient but obvious prototype: better to factor out field
		* handlers (ie ref, primintive and this one), to farm out into appropriate
		* arrays for each.
		*	"Children" def: existing (ie non-null) classes (ie non-primitive).
		**/
	private boolean hasChildren(Object o) {
		if (objStore.size()>0)	//consider: return in fields - neat return
			return true;
		try {
			Field[] fields = getAllFields(o, false);	//later call!
			//Field[] fields = o.getClass().getFields();
			for (int i=0; i<fields.length; i++) {
				Field f = fields[i];
				if ((f.getModifiers() & NOT_SERIALIZED) != 0)
					continue;
				if (!f.getType().isPrimitive() && f.getType()!=String.class) 
					if (f.get(o)!=null)
						return true;
			}
			return false;		//found no non-primitive (includes String),
				//non-null objects within it	
		}
		catch (Exception e) {System.err.println(e); System.exit(0);}
		return false;	//unreachable code
	}


	/**--------------------------------------------------------------------------
		* serializeFields
		* ---------------
		* SUGGEST: abstracting a "printAttr(String name, String value)" method
		**/
	private void serializeFields(Object o, String indent) {
		if (o==null) return;
		Field[] fields = getAllFields(o, true);	//primitive: 1st call
		//Field[] fields = o.getClass().getFields();
		//Field[] fields = o.getClass().getDeclaredFields();
			//getDeclaredFeilds doesn't return inherited fields.
				//NOTE!!!  getFields() returns ONLY public fields
				//getDeclaredFields() returns everything - except inherited fields
		try {
			for (int i=0; i<fields.length; i++) {
				Field f = fields[i];
				String fieldName = shadowEncode(f, o.getClass());	//added for shadowing
				if (DEBUG) System.err.println("Field "+i+": "+f);
				if ((f.getModifiers() & NOT_SERIALIZED) != 0)
					continue;	//skip transient and static fields
				Class type = f.getType();
						//how bad would it be to treat these two following cases the same?
						//ie. and *always* do encodeXML() automatically, in the ParserXML
				if (DEBUG) System.err.println("here1");
				if (type.isPrimitive())
				{
					out.println();							//Attribute indenting
					out.print(indent);		//indent each attribute
					printAttr(fieldName, f.get(o)+"");
				}
				else if (f.get(o)!=null && type==String.class)	//.getName().equals("java.lang.String"))
				{
					out.println();							//Attribute indenting
					out.print(indent);		//indent each attribute
				if (DEBUG) System.err.println("here2");
					//fieldName;
				if (DEBUG) System.err.println("here3");
					//f.get(o);
				if (DEBUG) System.err.println("here4");
					if (putAlias(f.get(o), shadowEncode(f, o.getClass()), true, indent)) {
							//only inline if not done already
							//problem for attributes in String
						printAttr(fieldName, encodeXML(f.get(o)+""));
						removeAlias(f.get(o));
					}
				}
					//NB: if neither String nor primitive, it just falls right through
			}
		} catch (Exception e) {
			System.out.println("Field: "+e);
			e.printStackTrace();
			System.exit(1);
		}
				//(3). process returned values.
		int i=0;
		for (Enumeration keys=primStore.elements(); keys.hasMoreElements(); i++) {
			out.println();							//Attribute indenting
			out.print(indent+SPACER);		//indent each attribute
			printAttr(INTERNAL_PRIMITIVE_TOKEN+i, encodeXML((String)keys.nextElement()));
			//NOTE: will never be String, because it is serialized using object.
		}
		primStore.clear();	//NB: reset when consumed
			//consider: can reuse, because never recurse. Different from objStore
		//NB: need to explicitly cast Object to String
		//Consider: if an array, could specify a String base type
		//NB: lazy for now: always encode as XML, even tho only String needs it
	}


/**--------------------------------------------------------------------------
	* isShadowed
	* ----------
	* Inherited fields with the same name as present fields are "shadowed",
	* and JSX needs some way to distinguish them.  Needed for both primitives
	* and objects (references).
	* We check to see if there is a field of the same name in a younger (sub)
	* class.  The order we scan the descendents does not matter - thus we can
	* use superclass() (no getSubclass() is avail!), and start at the bottom,
	* and work our way up until we are one class away from the class declaring
	* the field.
	* A
	* |
	* B  <-- field declared here: B.class.getDeclaredField(fieldName);>
	* |
	* C
	* |
	* D 	<-- the class of the object we have: me.getClass();
	* |
	* E
	*
	* Attribution: based on scratch code by Wes Biggs, 20 March 2001, which
	* did the opposite - if a given field shadowed another. */
	boolean isShadowed(Field f, Class c) {
		Class stopc = f.getDeclaringClass();
		for (; c!=stopc; c=c.getSuperclass()) {
  		try {
    		Field f2 = c.getDeclaredField(f.getName());
 				//System.out.println("Given field '" +qualField(f)+ "'");
 				//System.out.println("is SHADOWED BY '" +qualField(f2)+ "'");
				return true;
  		} catch (NoSuchFieldException e) {
    		// expect this in most cases
  		}
		}
 		//System.out.println("not shadowed");
		return false;
	}
/**--------------------------------------------------------------------------
	* qualField
	* ---------
	* This creates the class-qualified field name.  Arguably, this and the
	* XMLDeserialize "parseQualField" could go into the ParseUtilities class.
	**/
	String qualField(Field f) {
		return f.getDeclaringClass().getName()+"."+f.getName();
	}

/**--------------------------------------------------------------------------
	* shadowEncode
	* ------------
	* Returns the qualified field name, if needed; else just the plain name.
	**/
	String shadowEncode(Field f, Class c) {
		if (isShadowed(f, c))
			return qualField(f);
		else
			return f.getName();
	}


	/** This should likewise be in ParserXML.
		*/
	private void printAttr(String attrName, String value) {
		//out.print(SPACER +attrName+ " = \"" +value+ "\"");
		out.print(" "+ attrName+ "=\"" +value+ "\"");
	}

	/** And this too (should be in ParserXML).
		* Change to StringBuffer based on Raimi's "non-fork" version.
		*/
	private String encodeXML(String in) {
		StringBuffer out = new StringBuffer(in.length());	//cap at least this long

	//FIXME: \ u encoding <-- compiler picks it up as a malformed unicoding!
	//is \u0000 a legitimate char? Can it be rep in XML?
		for (int i=0; i<in.length(); i++) {
			char c = in.charAt(i);
			switch (c) {
				case '&': out.append("&amp;"); break;
				case '<': out.append("&lt;"); break;
				case '>': out.append("&gt;"); break;
				case '\'': out.append("&apos;"); break;
				case '"': out.append("&quot;"); break;
				case '\\': out.append("\\\\"); break;	//escape the escape char...
				case '\t': case '\n': case '\r': out.append(c); break;
				default:	//NB: exceptions done already (0x9 \t, 0xA \n, 0xD \r)
					//TODO: What about other ranges??
					if (c<=0x1F) {	//char is unsigned, so no problem with -ve rep
 						out.append( "\\u" + Integer.toHexString(c+0x10000).substring(1) );
							//must be exactly 4 char long: add leading 0's, remove "1"
					} else {
 						out.append(c);
					}
			}
		}
		return out.toString();	//fast: a ref, not copy (unless needed).
	}


	/**--------------------------------------------------------------------------
		* serializeReferences(o," ")
		* -------------------
		**/
	//should factor out the two iterations over fields (attr and ref)
	private void serializeReferences(Object o, String indent) throws IOException {
		if (o==null) return;
		Field[] f = getAllFields(o, false);	//ref: 2nd call
		//Field[] f = o.getClass().getFields();
			//Field[] f = o.getClass().getDeclaredFields(); //skips inherited fields
		for (int i=0; i<f.length; i++) {
			//bug found by Romain - thanks!
			if ((f[i].getModifiers() & NOT_SERIALIZED) != 0)//left this out before!
				continue;
			Class type = f[i].getType();
			try {
				if (!(type.isPrimitive() || type==String.class))		//.getName().equals("java.lang.String")))
					serializeObject(f[i].get(o), shadowEncode(f[i], o.getClass()), indent+SPACER);
			} catch (Exception e) {System.err.println("References: " +e);}
		}
	}


	/**--------------------------------------------------------------------------
		* getAllFields(o,true)
		* ------------
		*	Walks up the class hierarchy, getting:
		*		- non-public (private etc)
		*		- inherited fields;
		* NB: getDeclaredFields() does only the former, getFields() only the latter.
		* Alt implementation:
		*		2 pass: build a vector of arrays, then arraycopy to the correct size
		* firstCall: called for both primitives and references.  But should
		* invoke internal serialization only once (on firstCall).  NB always need
		* to check for writeObject(), to know whether to skip (ie default) or not.
		**/
		Vector defaultCache;	//still is recursion
	private Field[] getAllFields(Object o, boolean firstCall) {
		Class c = o.getClass();
		Vector v = new Vector();
		int cacheIndex = 0;
		if (CACHE_DEBUG)
			System.err.println("Entering   " +firstCall);
		if (firstCall) {
			objStore = new Vector(); 	//NB: once for all superclasses: collect all
			defaultCache = new Vector();
		}

		//TEST: do just for leaf class. (later will do for all)
		//Need to go in reverse order (superclass -> subclass), because that is
		//the order that Serialization does it, and which classes which customise
		//their own serialization expect.  Thanks Mark Collette (Simulutions.com)!
		//SOLN: create an array, then step it backwards.
		Class[] superClasses = getReversedSuperClasses(c);

		//old: do {
		for (int j=0; j<superClasses.length; j++) {  //traverse root to leaf
				//c = superClasses[(superClasses.length-1)-j];	//... from root (but for Object) to leaf
			c = superClasses[j];	//... from root (but for Object) to leaf
			if (CACHE_DEBUG)
				System.err.print(o.getClass() +"CACHE INDEX: " +cacheIndex+"\t\t");
			if (firstCall) {	//do for each superclass
				internalSer(c, o); //get returned values in caller (confusing?)
				defaultCache.add(new Boolean(defaultFlag));	//use array for param types?
				if (CACHE_DEBUG)
					System.err.println("firstcall: CACHE: " +defaultCache);
			} else {
				if (CACHE_DEBUG)
					System.err.println("not firstcall CACHE: " +defaultCache);
				defaultFlag = ((Boolean)defaultCache.get(cacheIndex)).booleanValue();
						//retrieve from cache for later calls
			}
			boolean doesOwn = doesOwnSer(c);
			//(1). if it doesn't do internal serialization, then serialize normally
			//(2). if it does internal serialization, only serialize if default called
				//GREAT!  Default fields omitted if defaultWriteObject() not called.
			if (!doesOwn || (doesOwn && defaultFlag)) {	//verbose is clearer
					Field[] f = c.getDeclaredFields();
					for (int i=0; i<f.length; i++)
						v.add(f[i]);
			}
			cacheIndex++;
		}	//old: while ( (c = c.getSuperclass()) != Object.class);	//not 'C'lass

		Field[] f = new Field[v.size()];	//scope of other f finished
		v.copyInto(f);		//obviate casting; and only way avail in 1.1
		//Java 1.2 security MAGIC!
		Field.setAccessible(f, true);	//new in 1.2 - need to do version checks
		return f;
	}

/**--------------------------------------------------------------------------
	* getReversedSuperClasses
	*	-----------------------
	* A utility class, common to both serialization and deserialization
	* - need to factor out into a utility class.
	* used by both XMLSerialize and XMLDeserialize
	**/
	static public Class[] getReversedSuperClasses(Class c) {
		//System.err.println("class c = " + c);
		//
		if (c==Object.class)
			return new Class[0];	//Object has no superclasses - upset caller?
		Class storeC = c;
		int classDepth = 0;	//count for array length (faster than using Vector)
		do { classDepth++; } while ( (c = c.getSuperclass()) != Object.class);
			//note: we don't count "Object" root, b/c not serialized (no state).
		if (DEBUG) System.err.println("classDepth = " + classDepth);
		Class[] superClasses = new Class[classDepth];	//of correct length
		c = storeC; 	//restore
		int iSuper=superClasses.length-1;	//create it in reverse
		do { superClasses[iSuper--] = c; } while ( (c = c.getSuperclass()) != Object.class);
			//hmmm... nicer to start at "length", and use "--i"?  Then ends on "0"
			//in order from root to leaf:
				//[0] is the last ancestor before Object; (ie the root but for Object)
				//[last] is the leaf - the actual class of the object
		if (DEBUG) {
			for (int j=0; j<superClasses.length; j++)
				System.err.println("superClasses["+j+"] = " + superClasses[j]);
		}
		return superClasses;
	}


	/**-------------------------------------------------------------------------
		*	internalSer
		* -----------
		* returned values are stored in:
		*		- defaultFlag
		*		- objStore
		*		- primStore
		*	  - returns: whether o implements its own serialization
		* NB: With new approach to serialization, this invocation will
		* have the effect of depositing the written objects to the objStore,
		* where they will be later retrieved: a crude, one-level stack for
		*	return values.
		*
		* AH!  This is why class and object are both needed: the class forms
		* view onto the object.  Different slices are possible, for each
		* superclass...
		**/
	private boolean internalSer(Class clazz, Object o) {
			//Class clazz = o.getClass();
			defaultFlag = false;	//...it can be set within the writeObject()
		Method m;
			if ((m=getDeclaredMethod(clazz, "writeObject", OOS_ARGS,
					Modifier.PRIVATE, Modifier.STATIC)) != null) {
			if (INTERNAL_DEBUG)
				System.err.print("--JSX serialize--");
			//invokeObjectWriter(m, o);
	//NB: return values are cleared when read (consumed)
	//Consider: reset when they are not generated?
			try {
	//(2). gather (catch) returned values (in a bucket).
				if (INTERNAL_DEBUG)
					System.err.println("invoking: " +m);
				m.invoke(o, writeObjectArglist);	//might as well do them all!
				//objStore are in effect returned in objStore (returnedObjs?) */
				//System.err.println("invoking: " +m);
			} catch (Exception e) {
				System.err.println(e);
				e.printStackTrace();
			}
			//NB: Only process if they exist...	hmmm, auto, if size = 0
			if (INTERNAL_DEBUG)
				System.err.println("default write: "+defaultFlag);	//check
			if (INTERNAL_DEBUG)
				System.err.println("Object Store: "+objStore);	//check
			if (INTERNAL_DEBUG)
				System.err.println("Primitive Store: "+primStore);	//check
			/* now, we write them all out individually (not the Vector!) */
			return true; //"it implements writeObject :-)"
		}
		else {	
			defaultFlag = true;	//if not writeObject(), then do defaults
			return false;	//"it doesn't implement writeObject :-("
		}
	}

	/**-------------------------------------------------------------------------
		*	doesOwn
		* -------
 		* Need to check the same superclass three times (refs, fields, hasChild).
		* If called internSer, would collect three lots of information!
		* We can't cache it easily, because there are typically several
		* superclasses for each object.
		* The only downside of this approach is the inefficiency.  However, the
		* way to counteract this is to refactoring the three calls into one.
		**/
	private boolean doesOwnSer(Class clazz) {
		return ((getDeclaredMethod(clazz, "writeObject", OOS_ARGS,
			Modifier.PRIVATE, Modifier.STATIC)) != null);
	}




	/**--------------------------------------------------------------------------
		* Vector (container type)
		* ------
		* Contains objects only (primitives written as Wrappers inc String)
		* spc used instead of indent - is there any reason for this?
		*
		* Note: Does not provide extensibility by allowing subclasses to override
		* this method of Vector serialization (and Hashtable, array, etc).
		* Static methods cannot be overridden.  To provide this, a instance
		* would need to be created at some stage - perhaps lazy instantiation
		* when first called.  Then, all such method should be made object methods
		* (ie not static), so that they can be overridden.
		**/
	private void serializeVectorReferences (Vector o, String indent) throws IOException{
		final String spc = indent + SPACER;
		if (o==null) throw new RuntimeException("impossible; redundant safety");
		for (Enumeration keys=o.elements(); keys.hasMoreElements(); ) {
			Object obj = keys.nextElement();
			serializeWrapperOrObject(obj, spc);
			//if (keys.hasMoreElements()) out.println();		//separating blank line
		}
	};		//what's the idea with the trailing ";"?



	/**--------------------------------------------------------------------------
		* Wrapper OR ObjectVector
		* -----------------------
		* A helper method (used for Vector and Array only so far), where the
		* object is assumed to have no "field name".
		**/
	private void serializeWrapperOrObject (Object obj, String spc) throws IOException{
		if (obj==null) {
			printOpenTag(NULL_TOKEN, spc);
			out.println("/>");		//good to have control of the newline here
			return;
		}
		Class type = obj.getClass();
		if (type==Boolean.class ||		//is there a cleverer way to do this?
				type==Byte.class ||				//we could loop, yes, but...
				type==Short.class ||
				type==Integer.class ||
				type==Long.class ||
				type==Float.class ||
				type==Double.class)
			serializeWrapper(obj, null, spc);
		else if (type==String.class ||
				type==Character.class)
			serializeWrapper(obj, null, spc);	//will cast do Character?
		else
			serializeObject(obj, null, spc);	//null== no name
	}

	/**--------------------------------------------------------------------------
		* Hashtable (container type)
		* ---------
		* Each key and value is an object.
		* distinguished by position alone.
		* primitives type written as special "wrapper" objects (String, too).
		**/
	private void serializeHashtableReferences (
			Hashtable o, String indent) throws IOException{
		if (o==null) throw new RuntimeException("impossible; redundant safety");
		for (Enumeration keys=o.keys(); keys.hasMoreElements(); ) {
			Object key = keys.nextElement();
			Object value = o.get(key);
			Object[] objs = {key,        value};
			String[] spc =  {indent+SPACER, indent+SPACER};	//so can stagger, if we want
			for (int i=0; i<objs.length; i++) {
				Class type = objs[i].getClass();
				if (type==Boolean.class ||		//is there a cleverer way to do this?
						type==Byte.class ||				//we could loop, yes, but...
						type==Short.class ||
						type==Integer.class ||
						type==Long.class ||
						type==Float.class ||
						type==Double.class)
					serializeWrapper(objs[i], null, spc[i]);
				else if (type==String.class ||
						type==Character.class)
					serializeWrapper(objs[i], null, spc[i]);
				else
					serializeObject(objs[i], null, spc[i]);	//null== no name
			}
			if (keys.hasMoreElements()) out.println();		//separating blank line
		}
	}

	/** -------------------------------------------------------------------------
		* serializeWrapper()
		* ------------------
		* Note: must check aliasing, *before* encodeXML creates a new object...!
		*
		*	Is this where we should put the name, for case of root wrapper?
		* We know it must be a String in this case; but how about a more general
		*	wrapper? 
		*
		**/
	private void serializeWrapper(Object value, String name, String indent) {
		if (DEBUG_HASH) System.err.println("----enter serializeWrapper with "+name+"="+value);
		String tag = value.getClass().getName();		//"java.lang.String";
		if (putAlias(value, name, false, indent)) { //any reason for null?
			//true returned: (first time seen - an alias was put)
			if (DEBUG_HASH) System.err.println("found NO alias, so do inline (in serializeWrapper)");
			printOpenTag(tag, indent);
			if (name!=null)
				printAttr(NAME_TOKEN, name);
			if (value.getClass()==String.class) //instanceof OK, as String is final
				printAttr(VALUE_TOKEN, encodeXML((String)value));
			else if (value.getClass()==Character.class) {
				printAttr(VALUE_TOKEN, encodeXML(value+""));
			}
			else {
				printAttr(VALUE_TOKEN, value+"");
			}
			out.print("/>");		//good to have control of the newline here
				//move into ParserXML?
			//printCloseTag(tag, "");
			//NOTE: a wrapper *always* is an empty tag, because it can't have children
			out.println();	//instead of printCloseTag()
			removeAlias(value);
		}
		else {
				//false returned
			if (DEBUG_HASH) System.err.println("found object aliased already (in serializeWrapper)");
			//putAlias() takes care of this already
		}
	}
		
	private void serializeClass(Class clazz, String name, String indent) {
		String tag = "java.lang.Class";
		if (putAlias(clazz, name, false, indent)) {
			//true returned: (first time seen - an alias was put)
			printOpenTag(tag, indent);
			if (name!=null)
				printAttr(NAME_TOKEN, name);
			//if (putAlias(clazz.getName(), CLASSNAME_TOKEN, true, indent))
				//comment out, because this invented field shouldn't be aliases
			printAttr(CLASSNAME_TOKEN, clazz.getName());
			out.print("/>");		//good to have control of the newline here
			out.println();	//instead of printCloseTag()
			removeAlias(clazz);
		}
	}


/** ===========================================================================
	*	===========================================================================
	* helper code for writeObject()
	*	===========================
	**/
/**--------------------------------------------------------------------------
	* getDeclaredMethod
	*	-----------------
	* A utility class, common to both serialization and deserialization
	* - need to factor out into a utility class.
	*
	* 	CHANGED: "static" - we make it static for now
	* Taken directly from ObjectOutputStream...  too general, but a good start.
	**/
	//NB: default "package" accessibility, so XMLDeserialize can access it
	static Method getDeclaredMethod(Class clazz,
			String methodName, Class[] args, 
			int requiredModifierMask,
			int disallowedModifierMask) {
		Method method = null;
		try {
			method = 
				clazz.getDeclaredMethod(methodName, args);
			if (method != null) {
				int mods = method.getModifiers();
				if ((mods & disallowedModifierMask) != 0 ||
					(mods & requiredModifierMask) != requiredModifierMask) {
						method = null;
				} else {
					method.setAccessible(true);		//needed, because prviate
				}
			}
		} catch (NoSuchMethodException e) {
	    // Since it is alright if methodName does not exist,
	    // no need to do anything special here.
		}
		return method;
	}

	/*
	try {
		if ((m=testClazz.getDeclaredMethod("writeObject", new Class[] {})) != null)
			System.err.println("JSX1 "+m+ "Class " +testClazz
				+ " has a writeObject()");
		else
			System.err.println("JSX1.  Class " +testClazz+ " has no writeObject()");
	} catch (Exception e) {
			System.err.println(e);
	}

	*/


/**--------------------------------------------------------------------------
	* invokeObjectWriter
	*	------------------
	* 	CHANGED: "static" - we make it static for now
	* 	CHANGED: send in Method m!
	* Taken directly from ObjectOutputStream... good for exceptions!
	**/
	private void invokeObjectWriter(Method m, Object obj) throws IOException {
		try {
			m.invoke(obj, writeObjectArglist);
		} catch (InvocationTargetException e) {
			Throwable t = e.getTargetException();
			if (t instanceof IOException)
				throw (IOException)t;
			else if (t instanceof RuntimeException)
				throw (RuntimeException) t;
			else if (t instanceof Error)
				throw (Error) t;
			else
				throw new Error("internal error");
		} catch (IllegalAccessException e) {
	    /* Should not happen, unless there is a problem with the VM. This
	     * is because the writeObject method is obtained via
	     * ObjectStreamClass.getDeclaredMethod, which calls setAccessible
	     * on it, in privileged mode.
	     */
			throw new InternalError("Unable to access writeObject method");
		}
	}

}
