import java.io.*;

public class Utility
{
    /**
     * createFileReader.
     */
    static public BufferedReader createFileReader(String fileName)
        throws IOException
    {
        try {
            FileReader fr = new FileReader(fileName);
            BufferedReader bir = new BufferedReader(fr, 64000);
            return bir;
        }
        catch (IOException e) {
            System.err.println(e);
        }
        return null;
    }


    /**
     * createPrintWriter
     */
    static public PrintWriter createPrintWriter(String fileName)
        throws IOException
    {
        FileWriter fw = null;
        BufferedWriter bow = null;
        PrintWriter pow = null;
        String whichFile = null;

        try {
            fw = new FileWriter(fileName);
            bow = new BufferedWriter(fw, 6400);
            pow = new PrintWriter(bow);
        }
        catch (IOException e) {
            throw (e);
        }
        return pow;
    }

    /*
     * Used to create a big String by reading in a file
     */
    static public String createString(String fileName)
        throws IOException
    {
        StringBuffer sb = new StringBuffer(64000);
        String line;

        try {
            BufferedReader br = Utility.createFileReader(fileName);
            while ((line = br.readLine()) != null) {
                sb.append(line);
                sb.append("\n");
            }
        }
        catch (IOException e) {
            System.err.println(e);
        }
        finally {
            return sb.toString();
        }
    }

    /**
     * getDirectoryName
     * @param String fileName
     * given a fileName extract the directory portion (excluding the
     * trailing slash.
     * If not qualifed, return ""
     * e.g. /a/b/Foo.java returns "/a/b"
     *      Foo.java returns ""
     *      ../a/b/Foo.java returns "../a/b"
     *      ./a/b/Foo.java returns "./a/b"
     */
    public static String getDirectoryName(String fileName)
    {
        /*
         * get the corresponding fileDirectory for this fileName.
         */
        String fileDirectory = null;
        int lastSlash = fileName.lastIndexOf('/');
        if (lastSlash == -1) {
            fileDirectory = "";
        }
        else {
            fileDirectory = fileName.substring(0, lastSlash);
        }
        return fileDirectory;
    }


    /**
     * getBaseName
     * @param String fileName
     * accepts a leafName such as Foo.java and will return
     * the basename which would be Foo. Returns the original fileName
     * if there is no extension. Note that the extension is defined
     * as everything after the "."; the basename is everything before
     * the dot.
     * e.g. /a/b/Foo returns /a/b/Foo
     *      /a/b/Foo.java returns /a/b/Foo
     *      Foo.java returns Foo
     */
    public static String getBaseName(String fileName)
    {
        String baseName = null;
        int lastDot = fileName.lastIndexOf('.');
        if (lastDot == -1) {
            baseName = fileName;
        }
        else {
            baseName = fileName.substring(0, lastDot);
        }
        return baseName;
    }

    /**
     * getLeafName
     * @param String fileName
     * Will return the leafName of the provided fileName. A leaf name
     * is defined as everything after the last '/' in a fileName.
     * e.g. /a/b/Foo.java returns "Foo.java"
     *      /a/b/Foo returns "Foo"
     *      /a/b returns "b"
     *      /a/b/ returns ""
     *      Foo.java returns "Foo.java"
     */
    private static String getLeafName(String fileName)
    {
        String leafName = null;
        int lastSlash = fileName.lastIndexOf('/');
        if (lastSlash == -1) {
            leafName = fileName;
        }
        else if (lastSlash == fileName.length()) {
            leafName = "";
        }
        else {
            leafName = fileName.substring(lastSlash + 1);
        }
        return leafName;
    }

}


